#include "cellparameterswidget.h"
#include "ui_cellparameterswidget.h"

#include <QDoubleValidator>

extern "C" double CellVolumeForC(const double cell[6]);
extern "C" void GetCellCode(const char *spg, int lenspg, const double cell[6], int code[6]);

CellParametersWidget::CellParametersWidget(QWidget *parent) :
    QWidget(parent),    
    ui(new Ui::CellParametersWidget),
    cell{-1.0,-1.0,-1.0,-1.0,-1.0},
    cellVol(-1.0)
{
    ui->setupUi(this);

    QDoubleValidator *abcValidator = new QDoubleValidator(0.0, 10000.0, 8, this);
    ui->aLineEdit->setValidator(abcValidator);
    ui->bLineEdit->setValidator(abcValidator);
    ui->cLineEdit->setValidator(abcValidator);
    QDoubleValidator *angleValidator = new QDoubleValidator(0.0, 360.0, 6, this);
    ui->alphaLineEdit->setValidator(angleValidator);
    ui->betaLineEdit->setValidator(angleValidator);
    ui->gammaLineEdit->setValidator(angleValidator);

    connect(ui->aLineEdit, &QLineEdit::textEdited, this, [=](const QString &text){
        cell[0] = text.toDouble();
        updateVolume();
        if (cellcode[1] < 0) ui->bLineEdit->setText(text);
        if (cellcode[2] < 0) ui->cLineEdit->setText(text);
        emit cellParametersModified();
    });
    connect(ui->bLineEdit, &QLineEdit::textEdited, this, [=](const QString &text){
        if (cellcode[1] < 0) return;
        cell[1] = text.toDouble();
        updateVolume();
        emit cellParametersModified();
    });
    connect(ui->cLineEdit, &QLineEdit::textEdited, this, [=](const QString &text){
        if (cellcode[2] < 0) return;
        cell[2] = text.toDouble();
        updateVolume();
        emit cellParametersModified();
    });
    connect(ui->alphaLineEdit, &QLineEdit::textEdited, this, [=](const QString &text){
        cell[3] = text.toDouble();
        updateVolume();
        emit cellParametersModified();
    });
    connect(ui->betaLineEdit, &QLineEdit::textEdited, this, [=](const QString &text){
        cell[4] = text.toDouble();
        updateVolume();
        emit cellParametersModified();
    });
    connect(ui->gammaLineEdit, &QLineEdit::textEdited, this, [=](const QString &text){
        cell[5] = text.toDouble();
        updateVolume();
        emit cellParametersModified();
    });
}

CellParametersWidget::~CellParametersWidget()
{    
    delete ui;
}

void CellParametersWidget::updateVolume()
{
    if (anyIsEmpty()) {
        ui->volumeLabel->clear();
    } else {
        double cellv[6];
        getCellParameters(cellv);
        cellVol = CellVolumeForC(cellv);
        ui->volumeLabel->setText(QString::number(cellVol));
    }
}

void CellParametersWidget::setCell(const double cellpar[], const QString spg)
{
    for (int i=0; i < 6; i++) {
        cell[i] = cellpar[i];
        cellSav[i] = cellpar[i];
    }
    updateCell(spg);
}

void CellParametersWidget::getCellParameters(double cellp[]) const
{
    cellp[0] = ui->aLineEdit->text().toDouble();
    cellp[1] = ui->bLineEdit->text().toDouble();
    cellp[2] = ui->cLineEdit->text().toDouble();
    cellp[3] = ui->alphaLineEdit->text().toDouble();
    cellp[4] = ui->betaLineEdit->text().toDouble();
    cellp[5] = ui->gammaLineEdit->text().toDouble();
}

void CellParametersWidget::updateCell(const QString &spg)
{    
    GetCellCode(spg.toStdString().c_str(), spg.length(), cell, cellcode);
    setLineEdit();
    updateVolume();
}

void CellParametersWidget::hideRefineCheckBox()
{
    ui->refineCheckBox->hide();
}

double CellParametersWidget::cellVolume() const
{
    return cellVol;
}

bool CellParametersWidget::anyIsEmpty()
{
    if (ui->aLineEdit->text().isEmpty()) return true;
    if (ui->bLineEdit->text().isEmpty()) return true;
    if (ui->cLineEdit->text().isEmpty()) return true;
    if (ui->alphaLineEdit->text().isEmpty()) return true;
    if (ui->betaLineEdit->text().isEmpty()) return true;
    if (ui->gammaLineEdit->text().isEmpty()) return true;

    return false;
}

bool CellParametersWidget::isCellChanged()
{
    for (int i = 0; i < 6; i++) {
        if (cell[i] != cellSav[i]) return true;
    }
    return false;
}

bool CellParametersWidget::isCellRefined()
{
    return ui->refineCheckBox->isChecked();
}

void CellParametersWidget::setRefineEnabled(bool enable)
{
    ui->refineCheckBox->setEnabled(enable);
}

void CellParametersWidget::setLineEdit()
{
    if (cell[0] > 0) ui->aLineEdit->setText(QString::number(cell[0]));
    if (cellcode[1] < 0) {
        ui->bLineEdit->setEnabled(false);
        if (cell[0] > 0)
            ui->bLineEdit->setText(QString::number(cell[0],'f',7));
        else
            ui->bLineEdit->clear();
    } else {
        ui->bLineEdit->setEnabled(true);
        if (cell[1] > 0)
            ui->bLineEdit->setText(QString::number(cell[1],'f',7));
        else
            ui->bLineEdit->clear();
    }
    if (cellcode[2] < 0) {
        ui->cLineEdit->setEnabled(false);
        if (cell[0] > 0)
            ui->cLineEdit->setText(QString::number(cell[0],'f',7));
        else
            ui->cLineEdit->clear();
    } else {
        ui->cLineEdit->setEnabled(true);
        if (cell[2] > 0)
            ui->cLineEdit->setText(QString::number(cell[2],'f',7));
        else
            ui->cLineEdit->clear();
    }
    if (cellcode[3] < 0) {
        ui->alphaLineEdit->setEnabled(false);
        ui->alphaLineEdit->setText(QString::number(abs(cellcode[3])));
    } else {
        ui->alphaLineEdit->setEnabled(true);
        if (cell[3] > 0)
            ui->alphaLineEdit->setText(QString::number(cell[3],'f',6));
        else
            ui->alphaLineEdit->clear();
    }
    if (cellcode[4] < 0) {
        ui->betaLineEdit->setEnabled(false);
        ui->betaLineEdit->setText(QString::number(abs(cellcode[4])));
    } else {
        ui->betaLineEdit->setEnabled(true);
        if (cell[4] > 0)
            ui->betaLineEdit->setText(QString::number(cell[4],'f',6));
        else
            ui->betaLineEdit->clear();
    }
    if (cellcode[5] < 0) {
        ui->gammaLineEdit->setEnabled(false);
        ui->gammaLineEdit->setText(QString::number(abs(cellcode[5])));
    } else {
        ui->gammaLineEdit->setEnabled(true);
        if (cell[5] > 0)
            ui->gammaLineEdit->setText(QString::number(cell[5],'f',6));
        else
            ui->gammaLineEdit->clear();
    }
}
