#include "docdialog.h"
#include "ui_docdialog.h"
#include "nofocusdelegate.h"
#include "checkboxdelegate.h"
#include "floatdelegate.h"

DocDialog::DocDialog(const QVector<AtomInfo> &atoms, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::DocDialog)
{
    ui->setupUi(this);

    createAtomParModel(atoms);
    setAtomParTableView();
}

DocDialog::~DocDialog()
{
    delete ui;
}

void DocDialog::updateAtoms(QVector<AtomInfo> &atoms)
{
    for (int r = 0; r < atoms.size(); r++) {
        QModelIndex index = atomParModel->index(r,COL_OCC,QModelIndex());
        atoms[r].occ = index.data().toFloat();

        index = atomParModel->index(r,COL_BISO,QModelIndex());
        atoms[r].biso = index.data().toFloat();

        index = atomParModel->index(r,COL_BISOCODE,QModelIndex());
        atoms[r].bisoCode = index.data().toInt();

        index = atomParModel->index(r,COL_DOC,QModelIndex());
        atoms[r].doc = index.data().toInt();
    }
}

void DocDialog::headerCheckBoxClicked(bool state, int column)
{    
    for (int row = 0; row < atomParModel->rowCount(); row++) {
        QModelIndex index = atomParModel->index(row, column, QModelIndex());
        atomParModel->setData(index, QVariant(state?1:0));
    }
}

void DocDialog::checkBoxClicked(const QModelIndex &index, bool state)
{
    //If state is false uncheck the checkBoxHeader
    if (!state && cbHeader->isChecked(index.column())) {
        cbHeader->setChecked(index.column(), false);
        return;
    }

    //If state is true check the checkBoxheader if all checkboxes have been clicked
    if (state && !cbHeader->isChecked(index.column())) {
        if (rowCheckBoxClickedCount(index.column()) == atomParModel->rowCount())
            cbHeader->setChecked(index.column(), true);
    }
}

void DocDialog::createAtomParModel(const QVector<AtomInfo> &atoms)
{
    atomParModel = new QStandardItemModel(this);
    atomParProxyModel = new QSortFilterProxyModel(this);
    atomParProxyModel->setSourceModel(atomParModel);
    atomParModel->setColumnCount(5);
    const QStringList titles = {"Atom", "Occ.", "D.O.C.","B[iso]", "Refine B[iso]"};
    atomParModel->setHorizontalHeaderLabels(titles);

    atomParModel->setRowCount(atoms.size());
    for (int r = 0; r < atoms.size(); r++) {
        auto item = new QStandardItem(atoms.at(r).label);
        item->setFlags(item->flags() &  ~Qt::ItemIsEditable);
        atomParModel->setItem(r,COL_ATOM,item);

        item = new QStandardItem();
        item->setData(atoms.at(r).occ, Qt::DisplayRole);
        atomParModel->setItem(r,COL_OCC,item);

        item = new QStandardItem();
        item->setData(atoms.at(r).biso, Qt::DisplayRole);
        atomParModel->setItem(r,COL_BISO,item);

        item = new QStandardItem();
        item->setData(atoms.at(r).bisoCode, Qt::DisplayRole);
        atomParModel->setItem(r,COL_BISOCODE,item);

        item = new QStandardItem();
        item->setData(atoms.at(r).doc, Qt::DisplayRole);
        atomParModel->setItem(r,COL_DOC,item);
    }
}

void DocDialog::setAtomParTableView()
{
    ui->atomParTableView->setModel(atomParProxyModel);
    ui->atomParTableView->verticalHeader()->setDefaultSectionSize(22);
    ui->atomParTableView->verticalHeader()->hide();
    ui->atomParTableView->setShowGrid(false);
    ui->atomParTableView->setAlternatingRowColors(true);
    ui->atomParTableView->setSelectionMode(QAbstractItemView::SingleSelection);
    ui->atomParTableView->setSelectionBehavior(QAbstractItemView::SelectRows);
    ui->atomParTableView->horizontalHeader()->stretchLastSection();
    ui->atomParTableView->setItemDelegate(new NoFocusDelegate()); //to remove focus rectangle

    auto bisoCheckDelegate = new CheckBoxDelegate(ui->atomParTableView);
    bisoCheckDelegate->setColumn(COL_BISOCODE);
    ui->atomParTableView->setItemDelegateForColumn(COL_BISOCODE,bisoCheckDelegate);
    connect(bisoCheckDelegate, &CheckBoxDelegate::rowChecked, this, &DocDialog::checkBoxClicked);

    auto docCheckDelegate = new CheckBoxDelegate(ui->atomParTableView);
    docCheckDelegate->setColumn(COL_DOC);
    ui->atomParTableView->setItemDelegateForColumn(COL_DOC,docCheckDelegate);
    connect(docCheckDelegate, &CheckBoxDelegate::rowChecked, this, &DocDialog::checkBoxClicked);

    auto floatDelegate = new FloatDelegate(ui->atomParTableView);
    floatDelegate->setDecimals(5);
    ui->atomParTableView->setItemDelegateForColumn(COL_OCC, floatDelegate);
    ui->atomParTableView->setItemDelegateForColumn(COL_BISO, floatDelegate);

    cbHeader = new CheckBoxHeader(Qt::Horizontal, {COL_DOC,COL_BISOCODE}, true, ui->atomParTableView);
    ui->atomParTableView->setHorizontalHeader(cbHeader);
    cbHeader->setChecked(COL_DOC, false);
    cbHeader->setChecked(COL_BISOCODE, false);
    connect(cbHeader, &CheckBoxHeader::checkBoxClicked, this, &DocDialog::headerCheckBoxClicked);
    ui->atomParTableView->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);

    ui->atomParTableView->horizontalHeader()->setSortIndicator(-1, Qt::AscendingOrder); //avoid that setSortingEnabled force a sort
    ui->atomParTableView->setSortingEnabled(true);

    cbHeader->setChecked(COL_DOC, rowCheckBoxClickedCount(COL_DOC) == atomParModel->rowCount());
    cbHeader->setChecked(COL_BISOCODE, rowCheckBoxClickedCount(COL_BISOCODE) == atomParModel->rowCount());
}

int DocDialog::rowCheckBoxClickedCount(int col) const
{
    int count = 0;
    for (int r = 0; r < atomParModel->rowCount(); r++) {
        QModelIndex index = atomParModel->index(r,col,QModelIndex());
        if (index.data().toBool()) count++;
    }
    return count;
}
