#include "globaloptdialog.h"
#include "ui_globaloptdialog.h"
#include "dofwidget.h"
#include "mainwindow.h"
#include "crystaldata.h"
#include "missinginfodialog.h"
#include "qt_utils.h"

#include <QMessageBox>

const QStringList GlobalOptDialog::CFSTRING = {"R weighted profile", "R intensities", "R structure factor",
                                               "Chisquared 1", "Rwp 1", "R groups",
                                               "Chisquared 3", "Chisquared 4"};


extern "C" void get_gopt_info(c_gopt_info_type *info, sa_conditions *sainfo, bb_bc_Conditions *bbinfo, int ifButton);
extern "C" void finalize_gopt(c_gopt_info_type *info, sa_conditions *sainfo, bb_bc_Conditions *bbinfo);
extern "C" void c_evaluate_res(double res, int *numref, double *tthmaxr);
extern "C" void randomize_parameters();
extern "C" int crystal_number();
extern "C" void c_exit_sanneal();
extern "C" int is_missing_info();
extern "C" int is_missing_structure();

const int firstPhaseTab = 1;

GlobalOptDialog::GlobalOptDialog(MainWindow *mw) :
    QDialog(mw),
    ui(new Ui::GlobalOptDialog),    
    mWindow(mw),
    numPhase(0)
{
    ui->setupUi(this);

    ui->buttonBox->button(QDialogButtonBox::Apply)->setText("Run");
    ui->buttonBox->button(QDialogButtonBox::Apply)->setIcon(QIcon(":/pixmaps/pixmaps/go.png"));

    solDialog = new SolutionsDialog(mWindow);
    connect(solDialog, &SolutionsDialog::finished, this, &GlobalOptDialog::show);
    connect(this, &GlobalOptDialog::rejected, [this](){
        mWindow->clearStatusMessage();
        mWindow->restoreEnabledActions();
    });

    connect(mWindow,&MainWindow::mainWindowClosed,this,[=](){
        if (solDialog->isVisible()) solDialog->reject();
    });
}

GlobalOptDialog::~GlobalOptDialog()
{
    delete ui;
}


int GlobalOptDialog::setDialog(bool ifButton)
{
    if (is_missing_info()) {
        auto missInfoDialog = new MissingInfoDialog(this);        
        missInfoDialog->setDialog();
        int ret = missInfoDialog->exec();        
        if (ret == QDialog::Rejected) return 1;
        bool error = missInfoDialog->sendMissingInfo();
        if (error) return 2;
    }

    if (is_missing_structure()) {
        QString title = tr("Import starting model for simulated annealing");
        if (!mWindow->openImportStructure(title)) return 1;
    }

    get_gopt_info(&ginfo, &SACond, &BBCond, ifButton);

    setConditionsTab();

    setPhaseTab();

    ui->buttonBox->button(QDialogButtonBox::Apply)->setFocus();
    solDialog->resetLastSelection();

    mWindow->saveEnabledActions();
    mWindow->enableActions(MainWindow::DialogOpenAction);

    return 0;
}

void GlobalOptDialog::waitSignalIsEmitted()
{
    bool isWaitCursor = stopWaitCursor();
    QEventLoop loop;
    connect(this,&QDialog::finished,&loop,&QEventLoop::quit);
    //connect(mWindow,&MainWindow::mainWindowClosed,&loop,&QEventLoop::quit);
    loop.exec();
    if (isWaitCursor) QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
}

void GlobalOptDialog::reject()
{
    c_exit_sanneal();
    QDialog::reject();
}

void GlobalOptDialog::setConditionsTab()
{
    ui->CFComboBox->clear();
    for (int i=0; i < ginfo.ncfvet; i++) {
        ui->CFComboBox->addItem(CFSTRING.at(ginfo.cfvet[i]-1));
    }
    ui->CFComboBox->setCurrentText(CFSTRING.at(ginfo.cftype-1));
    ui->resDoubleSpinBox->setRange(ginfo.resmin, ginfo.resmax);
    ui->resDoubleSpinBox->setValue(ginfo.res);
    ui->randomSeedSpinBox->setValue(ginfo.seed);
    ui->nRunSpinBox->setValue(ginfo.nrun);
    if (ginfo.algo == SA_LOCAL) {
        ui->algorithmStackedWidget->setCurrentIndex(0);
        ui->autoMovesCheckBox->setChecked(SACond.autont);
        ui->nMovesSpinBox->setEnabled(!SACond.autont);
        ui->nMovesSpinBox->setValue(SACond.nt);
        ui->autoTempCheckBox->setChecked(SACond.autoti);
        ui->tempDoubleSpinBox->setEnabled(!SACond.autoti);
        ui->tempDoubleSpinBox->setValue(SACond.t_ini);
        ui->tempFactorDoubleSpinBox->setValue(SACond.temp_fact);
    } else {
        ui->algorithmStackedWidget->setCurrentIndex(1);
        ui->autoCyclesCheckBox->setChecked(BBCond.autont);
        ui->nCyclesSpinBox->setEnabled(!BBCond.autont);
        ui->nCyclesSpinBox->setValue(BBCond.niter);
    }
}

void GlobalOptDialog::getConditionsTab()
{
    //ginfo.cftype = ui->CFComboBox->currentIndex()+1;
    ginfo.cftype = ginfo.cfvet[ui->CFComboBox->currentIndex()];
    ginfo.res = ui->resDoubleSpinBox->value();
    ginfo.seed = ui->randomSeedSpinBox->value();
    ginfo.nrun = ui->nRunSpinBox->value();
    if (ginfo.algo == SA_LOCAL) {
        SACond.autont = myCheckState(ui->autoMovesCheckBox);
        SACond.nt = ui->nMovesSpinBox->value();
        SACond.autoti = myCheckState(ui->autoTempCheckBox);
        SACond.t_ini = ui->tempDoubleSpinBox->value();
        SACond.temp_fact = ui->tempFactorDoubleSpinBox->value();
    } else {
        BBCond.autont = myCheckState(ui->autoCyclesCheckBox);
        BBCond.niter = ui->nCyclesSpinBox->value();
    }
}

void GlobalOptDialog::setParameterNumber(int add)
{
    Q_UNUSED(add);

    int npar = 0;
    for (int i = 0; i < numPhase; i++) {
        DOFWidget *dofW = qobject_cast<DOFWidget *>(ui->globalOptTabWidget->widget(firstPhaseTab+i));
        npar += dofW->getNumberParam();
    }
    mWindow->getInfoTable()->updateValue(QString::number(npar),2);
}

void GlobalOptDialog::updatePhaseParameters()
{
    for (int i = 0; i < numPhase; i++) {
        DOFWidget *dofW = qobject_cast<DOFWidget *>(ui->globalOptTabWidget->widget(firstPhaseTab+i));
        dofW->updateComputingVariables();
    }
}

void GlobalOptDialog::setPhaseTab()
{
    int n = crystal_number();
    int numPhase0 = numPhase;
    if (n > numPhase) {
        for (int i=numPhase0; i < n; i++) {
            addPhaseTab();
        }
    } else if (n < numPhase) {        
        for (int i = n; i < numPhase0; i++) {
            removePhaseTab();
        }
    }

    for (int i = 0; i < numPhase; i++) {
        DOFWidget *dofW = qobject_cast<DOFWidget *>(ui->globalOptTabWidget->widget(firstPhaseTab+i));
        dofW->setWidget(i+1);
    }

    if (numPhase > 1) {
        for (int i = 0; i < numPhase; i++) {
            ui->globalOptTabWidget->setTabText(firstPhaseTab+i, crydata::crystalPhaseName(i+1));
        }
    } else {
        ui->globalOptTabWidget->setTabText(firstPhaseTab, "Crystal");
    }
}

void GlobalOptDialog::addPhaseTab(const QString &phaseName)
{
    DOFWidget *dofW = new DOFWidget();
    ui->globalOptTabWidget->addTab(dofW, phaseName);
    connect(dofW, &DOFWidget::parameterChecked, this, &GlobalOptDialog::setParameterNumber);
    numPhase++;
}

void GlobalOptDialog::removePhaseTab()
{
    QWidget *widget = ui->globalOptTabWidget->widget(numPhase);
    delete widget;
    numPhase--;
}

void GlobalOptDialog::on_resDoubleSpinBox_valueChanged(double arg1)
{
    int numref;
    double tthmaxr;
    c_evaluate_res(arg1, &numref, &tthmaxr);
    QString resText = QString("Nr. of reflections: %1 2%2: %3°").arg(numref).arg(QChar(0x03B8)).arg(tthmaxr);
    ui->resolutionLabel->setText(resText);
}

void GlobalOptDialog::on_autoCyclesCheckBox_clicked(bool checked)
{
    ui->nCyclesSpinBox->setEnabled(!checked);
}

void GlobalOptDialog::on_autoMovesCheckBox_clicked(bool checked)
{
    ui->nMovesSpinBox->setEnabled(!checked);
}

void GlobalOptDialog::on_autoTempCheckBox_clicked(bool checked)
{
    ui->tempDoubleSpinBox->setEnabled(!checked);
}

void GlobalOptDialog::on_buttonBox_clicked(QAbstractButton *button)
{
    QDialogButtonBox::StandardButton stdButton = ui->buttonBox->standardButton(button);

    if(stdButton == QDialogButtonBox::Apply) {
        getConditionsTab();
        this->hide();
#if !defined(JAV_WIDGET)
        mWindow->getJav()->raise();
#endif
        solDialog->resetLastSelection();
        updatePhaseParameters();
        QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
        finalize_gopt(&ginfo, &SACond, &BBCond);
        QApplication::restoreOverrideCursor();
        this->show();
        this->activateWindow();

    } else if (stdButton == QDialogButtonBox::Close) {
        c_exit_sanneal();
        // CrystalViewWidget *crv = mWindow->getJav();
        // crv->updateModify(1);
        //mWindow->getJav()->updateModify(1);

    } else if (stdButton == QDialogButtonBox::Help) {
        mWindow->openHelp("structure-solution-by-direct-space-methods");
    }
}

int GlobalOptDialog::myCheckState(const QCheckBox *checkBox)
{
    return checkBox->checkState() == Qt::Checked ? 1 : 0;
}

void GlobalOptDialog::on_randomizePushButton_clicked()
{
    updatePhaseParameters();
    randomize_parameters();
#if !defined(JAV_WIDGET)
    mWindow->getJav()->raise();
#endif
    for (int i = 0; i < numPhase; i++) {
        DOFWidget *dofW = qobject_cast<DOFWidget *>(ui->globalOptTabWidget->widget(firstPhaseTab+i));
        dofW->setWidget(i+1);  //TOFIX: should be better update only some widgets
    }
}

void GlobalOptDialog::on_pushButton_clicked()
{    
    int nsol;
    solDialog->setDialog(nsol, ginfo.nexport);
    if (nsol > 0) {
        this->hide();
        solDialog->show();
    } else
        QMessageBox::critical(this,"Error","No solutions available!");
}
