#include "intervalsdialog.h"
#include "ui_intervalsdialog.h"
#include "customplotzoom.h"

extern "C" int get_intervals_number();
extern "C" void get_intervals_limit(float tmin[], float tmax[]);
extern "C" void put_intervals_limit(float tmin[], float tmax[]);

IntervalsDialog::IntervalsDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::IntervalsDialog),
    intervalsChanged(false),
    moveCursor(true)
{
    ui->setupUi(this);

    connect(ui->buttonBox, &QDialogButtonBox::clicked, this, &IntervalsDialog::buttonBoxClicked);
    connect(ui->intervalsComboBox, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &IntervalsDialog::intervalSelected);
    connect(ui->minSpinBox, QOverload<double>::of(&QDoubleSpinBox::valueChanged), this, &IntervalsDialog::minSpinBoxChanged);
    connect(ui->maxSpinBox, QOverload<double>::of(&QDoubleSpinBox::valueChanged), this, &IntervalsDialog::maxSpinBoxChanged);
}

void IntervalsDialog::setIntervals(CustomPlotZoom *plot)
{
    mPlot = plot;
    connect(mPlot, &CustomPlotZoom::leftCursorPositionChanged, this, &IntervalsDialog::lcursorPositionChanged);
    connect(mPlot, &CustomPlotZoom::rightCursorPositionChanged, this, &IntervalsDialog::rcursorPositionChanged);

    int nIntervals = get_intervals_number();
    maxVal.resize(nIntervals);
    minVal.resize(nIntervals);
    auto tmin = new float[nIntervals];
    auto tmax = new float[nIntervals];

    get_intervals_limit(tmin, tmax);
    for (int i = 0; i < nIntervals; i++) {
        minVal[i] = tmin[i];
        maxVal[i] = tmax[i];
    }

    delete [] tmin;
    delete [] tmax;

    ui->intervalsComboBox->clear();
    ui->intervalsComboBox->addItem("Single Peak");
    for (int i = 1; i < nIntervals; i++) {
        ui->intervalsComboBox->addItem(QString("Interval n. %1").arg(i));
    }

    ui->intervalsComboBox->setCurrentIndex(1);
    intervalsChanged = false;
}

IntervalsDialog::~IntervalsDialog()
{
    delete ui;
}

void IntervalsDialog::intervalSelected(int index)
{
    if (index < 0) return;

    moveCursor = false;
    ui->minSpinBox->setValue(minVal.at(index));
    ui->maxSpinBox->setValue(maxVal.at(index));
    moveCursor = true;
    mPlot->setDoubleCursorPosition(minVal.at(index), maxVal.at(index));
}

void IntervalsDialog::buttonBoxClicked(QAbstractButton *button)
{
    QDialogButtonBox::StandardButton stdButton = ui->buttonBox->standardButton(button);

    if(stdButton == QDialogButtonBox::Ok) {
        if (intervalsChanged) {
            int nIntervals = minVal.size();
            auto tmin = new float[nIntervals];
            auto tmax = new float[nIntervals];
            for (int i = 0; i < nIntervals; i++) {
                tmin[i] = minVal[i];
                tmax[i] = maxVal[i];
            }
            put_intervals_limit(tmin, tmax);
            delete [] tmin;
            delete [] tmax;
            intervalsChanged = false;
        }
        setCloseDialog();        
    } else if (stdButton == QDialogButtonBox::Cancel) {

    }
}

void IntervalsDialog::minSpinBoxChanged(double value)
{
    int id = ui->intervalsComboBox->currentIndex();
    if (minVal.at(id) != static_cast<float>(value)) {
        if (moveCursor) mPlot->setDoubleCursorPosition(value, maxVal.at(id));
        intervalsChanged = true;
        minVal[id] = value;
    }
}

void IntervalsDialog::maxSpinBoxChanged(double value)
{
    int id = ui->intervalsComboBox->currentIndex();
    if (maxVal.at(id) != static_cast<float>(value)) {
        if (moveCursor) mPlot->setDoubleCursorPosition(minVal.at(id),value);
        intervalsChanged = true;
        maxVal[id] = value;
    }
}

void IntervalsDialog::lcursorPositionChanged(double pos)
{
    moveCursor = false;
    ui->minSpinBox->setValue(pos);
    moveCursor = true;
}

void IntervalsDialog::rcursorPositionChanged(double pos)
{
    moveCursor = false;
    ui->maxSpinBox->setValue(pos);
    moveCursor = true;
}

void IntervalsDialog::reject()
{
    setCloseDialog();

    QDialog::reject();
}

void IntervalsDialog::setCloseDialog()
{
    mPlot->deleteDoubleCursor();
    disconnect(mPlot, &CustomPlotZoom::leftCursorPositionChanged, this, &IntervalsDialog::lcursorPositionChanged);
    disconnect(mPlot, &CustomPlotZoom::rightCursorPositionChanged, this, &IntervalsDialog::rcursorPositionChanged);
}
