#include "mainwindow.h"
#include "commandline.h"
#include "expokeysettings.h"
#include "qt_utils.h"
#include "fileutils.h"
#if defined(JAV_WIDGET)
#include "libcomune.h"
#endif
#if USE_CONFIG_H
#include "config.h"
#endif

#include <QApplication>
#include <csetjmp>
#include <csignal>
#include <cstdlib>
#include <QMessageBox>
#include <QDebug>
#include <QMetaType>
#include <QDataStream>

extern "C" void  expo2005(ProgOptions *p, const char *filein, int lenIn, const char *fileout, int lenOut, const char *exepath, int lenPath, int *ier);
extern "C" int activate_mpi();

jmp_buf env;

void onSigabrt(int)
{
    longjmp (env, 1);
}

QCoreApplication *loadQt(int &argc, char **argv, int numTasks)
{
    QCoreApplication *application = NULL;

    if (numTasks == 1) {
        bool gui = true;
        for (int i = 1; i < argc; ++i) {
            if (!qstrcmp(argv[i], "--nogui"))
                gui = false;
        }

        if (gui)
        {
            if (setjmp(env) == 0)
            {
                signal(SIGABRT, &onSigabrt);
                application = new QApplication(argc,argv);
            }
            signal(SIGABRT, SIG_DFL);
            if (!application) qWarning() << qPrintable(formattedMessage("The Display cannot be accessed !!!\n"
                                                          "Program will go on without graphics", 50));
        }
    }
    if (!application)
        application = new QCoreApplication(argc,argv);
    return (application);
}

void test(MainWindow& win, ProgOptions &opt, const QString &testDir, const QString &filein, const QString &fileout, const QString& pathDataFiles, int &ier)
{
    QString filein0, fileout0;
    std::string exepath = pathDataFiles.toStdString();
    expo2005(&opt,filein0.toStdString().c_str(),filein0.length(),fileout0.toStdString().c_str(),
             fileout0.length(),pathDataFiles.toStdString().c_str(),exepath.length(), &ier);
    if (filein.isEmpty()) {
        //Run expo on all exp files in the test folder
        QString testFolder;
        if (testDir.isEmpty()) {
            testFolder = win.loadExamplesFolder();
        } else {
            testFolder = testDir;
        }
        QStringList files;
        fileutils::findFiles(files, testFolder, "*.exp");
        for (int i = 0; i < files.size(); i++) {
            QString filein = files.at(i);
            QString fileout = fileutils::removeExtension(filein) + ".out";
            win.loadExpFile(filein, fileout);
        }
        return;
    }

    //Run expo on filein 2 times
    //add extension '.exp' if not present
    if (!filein.endsWith(".exp", Qt::CaseInsensitive)) {
        filein0 = filein + ".exp";
    } else {
        filein0 = filein;
    }
    if (fileout.isEmpty()) {
        fileout0 = fileutils::removeExtension(filein0) + ".out";
    } else {
        fileout0 = fileout;
    }
    win.loadExpFile(filein0, fileout0);
    win.loadExpFile(filein0, fileout0);
}

int main(int argc, char *argv[])
{
    //Initialize MPI and evaluate number of tasks
    int nTasks = activate_mpi();

    QScopedPointer<QCoreApplication> a(loadQt(argc,argv,nTasks));
    bool isGuiApp = (qobject_cast<QApplication*>(QCoreApplication::instance())!=0);
    if(isGuiApp)
    {
#ifdef Q_OS_MACOS
    QApplication::setWindowIcon(QIcon(":/pixmaps/pixmaps/Expo.icns"));
#else
    QApplication::setWindowIcon(QIcon(":/pixmaps/pixmaps/expo.png"));
#endif
    }
#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
    qRegisterMetaTypeStreamOperators<_LightInfo>("_LightInfo");
#else
    qRegisterMetaType<_LightInfo>();
#endif

#ifdef Q_OS_WIN32
    QByteArray babel_dir = QCoreApplication::applicationDirPath().toUtf8();
    babel_dir.append("/../share/openbabel/data");    
    qputenv("BABEL_DATADIR", babel_dir);
#elif defined(Q_OS_MACOS)
    QFileInfo path_app(QCoreApplication::applicationDirPath());
    QByteArray contentsDir = path_app.absolutePath().toUtf8();
    QByteArray babel_dir = contentsDir + "/share/openbabel/data";
    qputenv("BABEL_DATADIR", babel_dir);
    QByteArray babel_lib = contentsDir + "/share/openbabel_formats";
    qputenv("BABEL_LIBDIR", babel_lib);
#endif
    qApp->setApplicationVersion(APP_VERSION);
    qApp->setApplicationName(APP_NAME);
    qApp->setOrganizationName("IC");
    qApp->setOrganizationDomain("www.ba.ic.cnr.it/softwareic");

    QLocale locale = QLocale::system();
    if (locale.decimalPoint() == ',') {
        QLocale::setDefault(QLocale::c());
        setlocale(LC_ALL,"C");
    }

    QCommandLineParser parser;
    parser.setApplicationDescription(QCoreApplication::translate("main","Program for structure solution process from powder diffraction data:"));
    QString errorMessage;
    ProgOptions opt;
    QString filein, fileout, testFolder;
    bool testApp;
    switch (parseCommandLine(parser, filein, fileout, opt, errorMessage, testApp, testFolder)) {
    case CommandLineOk:
        break;
    case CommandLineError:
        fputs(qPrintable(errorMessage), stderr);
        fputs("\n\n", stderr);
        fputs(qPrintable(parser.helpText()), stderr);
        return 1;
    case CommandLineVersionRequested:
    {
        AppVersionInfo appVersion = getVersionInfo();
        printf("%s Version %s\n%s\n%s\n%s\n",
               qPrintable(QCoreApplication::applicationName()),
               qPrintable(QCoreApplication::applicationVersion()),
               qPrintable(QString("Created on %1").arg(appVersion.data)),
               qPrintable(appVersion.compilersString),
               qPrintable(appVersion.qtVersionInfo));
    }
        return 0;
    case CommandLineHelpRequested:
        parser.showHelp();
        Q_UNREACHABLE();
    }

    //Set current folder from filein
    QString currentDir = fileutils::setCurrentDirFromFile(filein);
    if (!currentDir.isEmpty()) {
        QSettings settings;
        settings.setValue(DEFAULT_DIR_KEY,currentDir);
        filein = QFileInfo(filein).fileName();
    }

    //Find folder with program files
    QString errPath;
    QStringList dataFiles = {"syminfo.lib","AtomProperties.xen","expo.spg"};
    QString pathDataFiles = fileutils::getDirDataFiles(dataFiles,errPath);
    MainWindow::setPathDataFiles(pathDataFiles);

    //Force nogui option when graphic display is not available
    if (!qobject_cast<QApplication *>(a.data())) opt.nogui = 1;
    //Force nogui option when more than 1 task is available
    if (nTasks > 1) opt.nogui = 1;

    int ier;
    if (opt.nogui) {
        if (pathDataFiles.isEmpty()) {
            qCritical() << "Problem occurred: " << errPath;
            ier = 1;
        } else {
            std::string exepath = pathDataFiles.toStdString();
            expo2005(&opt,filein.toStdString().c_str(),filein.length(),fileout.toStdString().c_str(),
                     fileout.length(),pathDataFiles.toStdString().c_str(),exepath.length(), &ier);
        }
        return (ier ? EXIT_FAILURE : EXIT_SUCCESS);
    } else {        
        MainWindow w;
        if (!pathDataFiles.isEmpty()) {
            QString xen = pathDataFiles + "AtomProperties.xen";
            leggixen(&w, xen);
        }
        w.show();
        if (pathDataFiles.isEmpty()) {
            QMessageBox::critical(&w,"Problem Occurred",errPath,QMessageBox::Ok);
            return EXIT_FAILURE;
        } else {
            if (testApp) {
                opt.autom = 1;
                test(w, opt, testFolder, filein, fileout, pathDataFiles, ier);
            } else {
                std::string exepath = pathDataFiles.toStdString();
                QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
                expo2005(&opt,filein.toStdString().c_str(),filein.length(),fileout.toStdString().c_str(),
                         fileout.length(),pathDataFiles.toStdString().c_str(),exepath.length(), &ier);
                QApplication::restoreOverrideCursor();
            }
            if (opt.autom) return (ier ? EXIT_FAILURE : EXIT_SUCCESS);
            if (ier) return EXIT_FAILURE;
            //return (ier ? EXIT_FAILURE : EXIT_SUCCESS);
        }
        return a->exec();
    }
}
