#include "powderdatawidget.h"
#include "backgrounddialog.h"
#include "ui_powderdatawidget.h"
#include "mainwindow.h"

#include <QMetaEnum>

extern "C" void get_back_settings(BackgroundSettings *backSettings);

PowderDataWidget::PowderDataWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::PowderDataWidget),
    parameterChanged(false)
{
    ui->setupUi(this);

    auto floatValidator = new QDoubleValidator();
    floatValidator->setDecimals(6);
    ui->zeroLineEdit->setValidator(floatValidator);
    ui->transLineEdit->setValidator(floatValidator);
    ui->dispLineEdit->setValidator(floatValidator);

    connect(ui->zeroLineEdit, &QLineEdit::textEdited, this, &PowderDataWidget::setParameterChanged);
    connect(ui->transLineEdit, &QLineEdit::textEdited, this, &PowderDataWidget::setParameterChanged);
    connect(ui->dispLineEdit, &QLineEdit::textEdited, this, &PowderDataWidget::setParameterChanged);
    connect(ui->modifyBackgroundButton, &QPushButton::clicked, this, &PowderDataWidget::onModifyBackgroundButtonClicked);
}

PowderDataWidget::~PowderDataWidget()
{
    delete ui;
}

void PowderDataWidget::setData(const dataset_t &data, const BackgroundSettings &backSettings)
{
    parameterChanged = false;

    setBackgroundLabel(backSettings);
    ui->backCheckBox->setChecked(data.bcode);

    ui->zeroLineEdit->setText(QString::number(data.zero,'f',6));
    ui->zeroCheckBox->setChecked(data.zcode);

    ui->transLineEdit->setText(QString::number(data.stran,'f',6));
    ui->transCheckBox->setChecked(data.stcode);

    ui->dispLineEdit->setText(QString::number(data.sdisp,'f',6));
    ui->dispCheckBox->setChecked(data.sdcode);
}

void PowderDataWidget::getData(dataset_t &data)
{
    data.bcode = ui->backCheckBox->isChecked();

    data.zero = ui->zeroLineEdit->text().toFloat();
    data.zcode = ui->zeroCheckBox->isChecked();

    data.stran = ui->transLineEdit->text().toFloat();
    data.stcode = ui->transCheckBox->isChecked();

    data.sdisp = ui->dispLineEdit->text().toFloat();
    data.sdcode = ui->dispCheckBox->isChecked();
}

void PowderDataWidget::setBackgroundLabel(const BackgroundSettings &backSettings)
{
    static int enumIndex = BackgroundDialog::staticMetaObject.indexOfEnumerator("BackgroundType");
    QString bFunctionName = BackgroundDialog::staticMetaObject.enumerator(enumIndex).valueToKey(backSettings.btype);
    bFunctionName.replace('_',' ');
    if (backSettings.btype == BackgroundDialog::Chebyshev ||
            backSettings.btype == BackgroundDialog::Polynomial ||
            backSettings.btype == BackgroundDialog::Cosine_Fourier) {
        ui->backgroundLabel->setText(QString("Function Type: %1 (%2 coefficients)").arg(bFunctionName).arg(backSettings.ncoef));
    } else {
        ui->backgroundLabel->setText(QString("Function Type: %1").arg(bFunctionName));
    }
}

bool PowderDataWidget::hasParameterToRefine()
{
    if (ui->backCheckBox->isChecked()) return true;
    if (ui->zeroCheckBox->isChecked()) return true;
    if (ui->transCheckBox->isChecked()) return true;
    if (ui->dispCheckBox->isChecked()) return true;

    return false;
}

bool PowderDataWidget::hasParameterChanged()
{
    return parameterChanged;
}

void PowderDataWidget::setRefineEnabled(bool enable)
{
    ui->backCheckBox->setEnabled(enable);
    ui->zeroCheckBox->setEnabled(enable);
    ui->dispCheckBox->setEnabled(enable);
    ui->transCheckBox->setEnabled(enable);
}

void PowderDataWidget::onModifyBackgroundButtonClicked()
{
    auto backDialog = new BackgroundDialog(this);
    backDialog->setBackground();
    int ret = backDialog->exec();
    if (ret == QDialog::Accepted) {
        BackgroundSettings backSettings;
        get_back_settings(&backSettings);
        setBackgroundLabel(backSettings);
    }
}

void PowderDataWidget::setParameterChanged()
{
    parameterChanged = true;
}
