MODULE SCATT_PARAMS

   use type_constants, only: DP

   implicit none

   private 

   integer, parameter, public :: SCATT_PAR_KIRKLAND = 1, SCATT_PAR_LOBATO = 2, &
                                 SCATT_PAR_DOYLE_TURNER = 3

   integer, parameter :: NPARKIRK = 12
   integer, parameter :: NPARLOBA = 10
   integer, parameter :: NPARDT   = 8

   interface fact_electron
   module procedure fact_electron_s, fact_electron_v
   end interface

   public fact_electron, get_electron_params

contains

   real(DP) function fact_electron_s(par,q2,scatt_par_type) result(fe)
!
!  return the electron scattering factor for atomic
!  q2  = q*q where q =1/d = sin(theta)/lambda (in 1/A)
!
   real(DP), dimension(:), intent(in) :: par
   real(DP), intent(in)               :: q2
   integer, intent(in)                :: scatt_par_type
!
   select case(scatt_par_type)
     case (SCATT_PAR_KIRKLAND)
           fe = fact_electron_kirkland_s(par,q2)
     case (SCATT_PAR_LOBATO)
           fe = fact_electron_lobato_s(par,q2)
     case (SCATT_PAR_DOYLE_TURNER)
           fe = fact_electron_doyle_turner_s(par,q2)
   end select
!
   end function fact_electron_s

!----------------------------------------------------------------------------------------------------------------

   function fact_electron_v(par,q2,scatt_par_type) result(fe)
   real(DP), dimension(:), intent(in) :: par
   real(DP), dimension(:), intent(in) :: q2
   integer, intent(in)                :: scatt_par_type
   real(DP), dimension(size(q2))      :: fe
   integer                            :: i

   do i=1,size(q2)
      fe(i) = fact_electron_s(par,q2(i),scatt_par_type)
   enddo

   end function fact_electron_v

!----------------------------------------------------------------------------------------------------------------
   real(DP) function fact_electron_kirkland_s(par,q2) result(fe)
!
!  return the electron scattering factor for atomic
!  q2  = q*q where q =1/d = sin(theta)/lambda (in 1/A)
!
   real(DP), dimension(NPARKIRK), intent(in) :: par
   real(DP), intent(in)                      :: q2
   integer                                   :: i
   integer, parameter                        :: NL = NPARKIRK/2
   
   fe = 0.0
!
!  Lorenztians
   do i=1,NL,2
      fe = fe + par(i) / (q2 + par(i+1))
   enddo
!
!  Gaussians
   do i=NL+1,NPARKIRK,2
      fe = fe + par(i) * exp(-par(i+1)*q2)
   enddo

   end function fact_electron_kirkland_s

!----------------------------------------------------------------------------------------------------------------

   real(DP) function fact_electron_lobato_s(par,q2) result(fe)
!
!  return the electron scattering factor for atomic
!  q2  = q*q where q =1/d = sin(theta)/lambda (in 1/A)
!
   real(DP), dimension(NPARLOBA), intent(in) :: par
   real(DP), intent(in)                      :: q2
   integer, parameter                        :: NP = NPARLOBA/2
   real(DP), dimension(NP)                   :: q2_par
   integer                                   :: i

   q2_par(:) = par(6:NPARLOBA)*q2
   fe = 0
   do i=1,NP
      fe = fe + par(i)*(2 + q2_par(i)) / (1 + q2_par(i))**2
   enddo

   end function fact_electron_lobato_s 

!----------------------------------------------------------------------------------------------------------------

   real(DP) function fact_electron_doyle_turner_s(par,q2) result(fe)
!
!  return the electron scattering factor for atomic
!  q2  = q*q where q =1/d = sin(theta)/lambda (in 1/A)
!
   real(DP), dimension(NPARDT), intent(in) :: par
   real(DP), intent(in)                    :: q2
   integer                                 :: i
   integer, parameter                      :: NL = NPARKIRK/2
   
   fe = 0.0
!
   do i=1,NPARDT,2
      fe = fe + par(i) * exp(-par(i+1)*q2)
   enddo

   end function fact_electron_doyle_turner_s

!----------------------------------------------------------------------------------------------------------------

   subroutine get_electron_params(zval,charge,fact,scatt_par_type,ier)
   integer, intent(in)                 :: zval
   integer, intent(in)                 :: charge
   real(DP), dimension(:), intent(out) :: fact
   integer, intent(in)                 :: scatt_par_type
   integer, intent(out)                :: ier
!
   ier = 1
   select case(scatt_par_type)
     case (SCATT_PAR_KIRKLAND)
         call get_kirkland_params(zval,fact,ier)
     case (SCATT_PAR_LOBATO)
         call get_lobato_params(zval,fact,ier)
     case (SCATT_PAR_DOYLE_TURNER)
         call get_doyle_turner_params(zval,charge,fact,ier)
   end select
!
   end subroutine get_electron_params

!----------------------------------------------------------------------------------------------------------------

   subroutine get_kirkland_params(zval,fact,ier)
!
!  Kirkland, E. J. (1998). Advanced Computing in Electron Microscopy.
!  New York: Springer. Appendix D and Chapter 5.
!
   integer, intent(in)                        :: zval
   real(DP), dimension(NPARKIRK), intent(out) :: fact
   integer, intent(out)                       :: ier
!
   ier = 0
!
!  Order of parameters:
!           a1              b1               a2               b2
!           a3              b3               c1               d1
!           c2              d2               c3               d3
   select case(zval)
   case(1)
      fact(:) = [ &
      4.20298324e-003, 2.25350888e-001, 6.27762505e-002, 2.25366950e-001,   &
      3.00907347e-002, 2.25331756e-001, 6.77756695e-002, 4.38854001e+000,   &
      3.56609237e-003, 4.03884823e-001, 2.76135815e-002, 1.44490166e+000]
   case(2)
      fact(:) = [ &
      1.87543704e-005, 2.12427997e-001, 4.10595800e-004, 3.32212279e-001,   &
      1.96300059e-001, 5.17325152e-001, 8.36015738e-003, 3.66668239e-001,   &
      2.95102022e-002, 1.37171827e+000, 4.65928982e-007, 3.75768025e+004]
   case(3)
      fact(:) = [ &
      7.45843816e-002, 8.81151424e-001, 7.15382250e-002, 4.59142904e-002,   &
      1.45315229e-001, 8.81301714e-001, 1.12125769e+000, 1.88483665e+001,   &
      2.51736525e-003, 1.59189995e-001, 3.58434971e-001, 6.12371000e+000]
   case(4)
      fact(:) = [ &
      6.11642897e-002, 9.90182132e-002, 1.25755034e-001, 9.90272412e-002,   &
      2.00831548e-001, 1.87392509e+000, 7.87242876e-001, 9.32794929e+000,   &
      1.58847850e-003, 8.91900236e-002, 2.73962031e-001, 3.20687658e+000]
   case(5)
      fact(:) = [ &
      1.25716066e-001, 1.48258830e-001, 1.73314452e-001, 1.48257216e-001,   &
      1.84774811e-001, 3.34227311e+000, 1.95250221e-001, 1.97339463e+000,   &
      5.29642075e-001, 5.70035553e+000, 1.08230500e-003, 5.64857237e-002]
   case(6)
      fact(:) = [ &
      2.12080767e-001, 2.08605417e-001, 1.99811865e-001, 2.08610186e-001,   &
      1.68254385e-001, 5.57870773e+000, 1.42048360e-001, 1.33311887e+000,   &
      3.63830672e-001, 3.80800263e+000, 8.35012044e-004, 4.03982620e-002]
   case(7)
      fact(:) = [ &
      5.33015554e-001, 2.90952515e-001, 5.29008883e-002, 1.03547896e+001,   &
      9.24159648e-002, 1.03540028e+001, 2.61799101e-001, 2.76252723e+000,   &
      8.80262108e-004, 3.47681236e-002, 1.10166555e-001, 9.93421736e-001]
   case(8)
      fact(:) = [ &
      3.39969204e-001, 3.81570280e-001, 3.07570172e-001, 3.81571436e-001,   &
      1.30369072e-001, 1.91919745e+001, 8.83326058e-002, 7.60635525e-001,   &
      1.96586700e-001, 2.07401094e+000, 9.96220028e-004, 3.03266869e-002]
   case(9)
      fact(:) = [ &
      2.30560593e-001, 4.80754213e-001, 5.26889648e-001, 4.80763895e-001,   &
      1.24346755e-001, 3.95306720e+001, 1.24616894e-003, 2.62181803e-002,   &
      7.20452555e-002, 5.92495593e-001, 1.53075777e-001, 1.59127671e+000]
   case(10)
      fact(:) = [ &
      4.08371771e-001, 5.88228627e-001, 4.54418858e-001, 5.88288655e-001,   &
      1.44564923e-001, 1.21246013e+002, 5.91531395e-002, 4.63963540e-001,   &
      1.24003718e-001, 1.23413025e+000, 1.64986037e-003, 2.05869217e-002]
   case(11)
      fact(:) = [ &
      1.36471662e-001, 4.99965301e-002, 7.70677865e-001, 8.81899664e-001,   &
      1.56862014e-001, 1.61768579e+001, 9.96821513e-001, 2.00132610e+001,   &
      3.80304670e-002, 2.60516254e-001, 1.27685089e-001, 6.99559329e-001]
   case(12)
      fact(:) = [ &
      3.04384121e-001, 8.42014377e-002, 7.56270563e-001, 1.64065598e+000,   &
      1.01164809e-001, 2.97142975e+001, 3.45203403e-002, 2.16596094e-001,   &
      9.71751327e-001, 1.21236852e+001, 1.20593012e-001, 5.60865838e-001]
   case(13)
      fact(:) = [ &
      7.77419424e-001, 2.71058227e+000, 5.78312036e-002, 7.17532098e+001,   &
      4.26386499e-001, 9.13331555e-002, 1.13407220e-001, 4.48867451e-001,   &
      7.90114035e-001, 8.66366718e+000, 3.23293496e-002, 1.78503463e-001]
   case(14)
      fact(:) = [ &
      1.06543892e+000, 1.04118455e+000, 1.20143691e-001, 6.87113368e+001,   &
      1.80915263e-001, 8.87533926e-002, 1.12065620e+000, 3.70062619e+000,   &
      3.05452816e-002, 2.14097897e-001, 1.59963502e+000, 9.99096638e+000]
   case(15)
      fact(:) = [ &
      1.05284447e+000, 1.31962590e+000, 2.99440284e-001, 1.28460520e-001,   &
      1.17460748e-001, 1.02190163e+002, 9.60643452e-001, 2.87477555e+000,   &
      2.63555748e-002, 1.82076844e-001, 1.38059330e+000, 7.49165526e+000]
   case(16)
      fact(:) = [ &
      1.01646916e+000, 1.69181965e+000, 4.41766748e-001, 1.74180288e-001,   &
      1.21503863e-001, 1.67011091e+002, 8.27966670e-001, 2.30342810e+000,   &
      2.33022533e-002, 1.56954150e-001, 1.18302846e+000, 5.85782891e+000]
   case(17)
      fact(:) = [ &
      9.44221116e-001, 2.40052374e-001, 4.37322049e-001, 9.30510439e+000,   &
      2.54547926e-001, 9.30486346e+000, 5.47763323e-002, 1.68655688e-001,   &
      8.00087488e-001, 2.97849774e+000, 1.07488641e-002, 6.84240646e-002]
   case(18)
      fact(:) = [ &
      1.06983288e+000, 2.87791022e-001, 4.24631786e-001, 1.24156957e+001,   &
      2.43897949e-001, 1.24158868e+001, 4.79446296e-002, 1.36979796e-001,   &
      7.64958952e-001, 2.43940729e+000, 8.23128431e-003, 5.27258749e-002]
   case(19)
      fact(:) = [ &
      6.92717865e-001, 7.10849990e+000, 9.65161085e-001, 3.57532901e-001,   &
      1.48466588e-001, 3.93763275e-002, 2.64645027e-002, 1.03591321e-001,   &
      1.80883768e+000, 3.22845199e+001, 5.43900018e-001, 1.67791374e+000]
   case(20)
      fact(:) = [ &
      3.66902871e-001, 6.14274129e-002, 8.66378999e-001, 5.70881727e-001,   &
      6.67203300e-001, 7.82965639e+000, 4.87743636e-001, 1.32531318e+000,   &
      1.82406314e+000, 2.10056032e+001, 2.20248453e-002, 9.11853450e-002]
   case(21)
      fact(:) = [ &
      3.78871777e-001, 6.98910162e-002, 9.00022505e-001, 5.21061541e-001,   &
      7.15288914e-001, 7.87707920e+000, 1.88640973e-002, 8.17512708e-002,   &
      4.07945949e-001, 1.11141388e+000, 1.61786540e+000, 1.80840759e+001]
   case(22)
      fact(:) = [ &
      3.62383267e-001, 7.54707114e-002, 9.84232966e-001, 4.97757309e-001,   &
      7.41715642e-001, 8.17659391e+000, 3.62555269e-001, 9.55524906e-001,   &
      1.49159390e+000, 1.62221677e+001, 1.61659509e-002, 7.33140839e-002]
   case(23)
      fact(:) = [ &
      3.52961378e-001, 8.19204103e-002, 7.46791014e-001, 8.81189511e+000,   &
      1.08364068e+000, 5.10646075e-001, 1.39013610e+000, 1.48901841e+001,   &
      3.31273356e-001, 8.38543079e-001, 1.40422612e-002, 6.57432678e-002]
   case(24)
      fact(:) = [ &
      1.34348379e+000, 1.25814353e+000, 5.07040328e-001, 1.15042811e+001,   &
      4.26358955e-001, 8.53660389e-002, 1.17241826e-002, 6.00177061e-002,   &
      5.11966516e-001, 1.53772451e+000, 3.38285828e-001, 6.62418319e-001]
   case(25)
      fact(:) = [ &
      3.26697613e-001, 8.88813083e-002, 7.17297000e-001, 1.11300198e+001,   &
      1.33212464e+000, 5.82141104e-001, 2.80801702e-001, 6.71583145e-001,   &
      1.15499241e+000, 1.26825395e+001, 1.11984488e-002, 5.32334467e-002]
   case(26)
      fact(:) = [ &
      3.13454847e-001, 8.99325756e-002, 6.89290016e-001, 1.30366038e+001,   &
      1.47141531e+000, 6.33345291e-001, 1.03298688e+000, 1.16783425e+001,   &
      2.58280285e-001, 6.09116446e-001, 1.03460690e-002, 4.81610627e-002]
   case(27)
      fact(:) = [ &
      3.15878278e-001, 9.46683246e-002, 1.60139005e+000, 6.99436449e-001,   &
      6.56394338e-001, 1.56954403e+001, 9.36746624e-001, 1.09392410e+001,   &
      9.77562646e-003, 4.37446816e-002, 2.38378578e-001, 5.56286483e-001]
   case(28)
      fact(:) = [ &
      1.72254630e+000, 7.76606908e-001, 3.29543044e-001, 1.02262360e-001,   &
      6.23007200e-001, 1.94156207e+001, 9.43496513e-003, 3.98684596e-002,   &
      8.54063515e-001, 1.04078166e+001, 2.21073515e-001, 5.10869330e-001]
   case(29)
      fact(:) = [ &
      3.58774531e-001, 1.06153463e-001, 1.76181348e+000, 1.01640995e+000,   &
      6.36905053e-001, 1.53659093e+001, 7.44930667e-003, 3.85345989e-002,   &
      1.89002347e-001, 3.98427790e-001, 2.29619589e-001, 9.01419843e-001]
   case(30)
      fact(:) = [ &
      5.70893973e-001, 1.26534614e-001, 1.98908856e+000, 2.17781965e+000,   &
      3.06060585e-001, 3.78619003e+001, 2.35600223e-001, 3.67019041e-001,   &
      3.97061102e-001, 8.66419596e-001, 6.85657228e-003, 3.35778823e-002]
   case(31)
      fact(:) = [ &
      6.25528464e-001, 1.10005650e-001, 2.05302901e+000, 2.41095786e+000,   &
      2.89608120e-001, 4.78685736e+001, 2.07910594e-001, 3.27807224e-001,   &
      3.45079617e-001, 7.43139061e-001, 6.55634298e-003, 3.09411369e-002]
   case(32)
      fact(:) = [ &
      5.90952690e-001, 1.18375976e-001, 5.39980660e-001, 7.18937433e+001,   &
      2.00626188e+000, 1.39304889e+000, 7.49705041e-001, 6.89943350e+000,   &
      1.83581347e-001, 3.64667232e-001, 9.52190743e-003, 2.69888650e-002]
   case(33)
      fact(:) = [ &
      7.77875218e-001, 1.50733157e-001, 5.93848150e-001, 1.42882209e+002,   &
      1.95918751e+000, 1.74750339e+000, 1.79880226e-001, 3.31800852e-001,   &
      8.63267222e-001, 5.85490274e+000, 9.59053427e-003, 2.33777569e-002]
   case(34)
      fact(:) = [ &
      9.58390681e-001, 1.83775557e-001, 6.03851342e-001, 1.96819224e+002,   &
      1.90828931e+000, 2.15082053e+000, 1.73885956e-001, 3.00006024e-001,   &
      9.35265145e-001, 4.92471215e+000, 8.62254658e-003, 2.12308108e-002]
   case(35)
      fact(:) = [ &
      1.14136170e+000, 2.18708710e-001, 5.18118737e-001, 1.93916682e+002,   &
      1.85731975e+000, 2.65755396e+000, 1.68217399e-001, 2.71719918e-001,   &
      9.75705606e-001, 4.19482500e+000, 7.24187871e-003, 1.99325718e-002]
   case(36)
      fact(:) = [ &
      3.24386970e-001, 6.31317973e+001, 1.31732163e+000, 2.54706036e-001,   &
      1.79912614e+000, 3.23668394e+000, 4.29961425e-003, 1.98965610e-002,   &
      1.00429433e+000, 3.61094513e+000, 1.62188197e-001, 2.45583672e-001]
   case(37)
      fact(:) = [ &
      2.90445351e-001, 3.68420227e-002, 2.44201329e+000, 1.16013332e+000,   &
      7.69435449e-001, 1.69591472e+001, 1.58687000e+000, 2.53082574e+000,   &
      2.81617593e-003, 1.88577417e-002, 1.28663830e-001, 2.10753969e-001]
   case(38)
      fact(:) = [ &
      1.37373086e-002, 1.87469061e-002, 1.97548672e+000, 6.36079230e+000,   &
      1.59261029e+000, 2.21992482e-001, 1.73263882e-001, 2.01624958e-001,   &
      4.66280378e+000, 2.53027803e+001, 1.61265063e-003, 1.53610568e-002]
   case(39)
      fact(:) = [ &
      6.75302747e-001, 6.54331847e-002, 4.70286720e-001, 1.06108709e+002,   &
      2.63497677e+000, 2.06643540e+000, 1.09621746e-001, 1.93131925e-001,   &
      9.60348773e-001, 1.63310938e+000, 5.28921555e-003, 1.66083821e-002]
   case(40)
      fact(:) = [ &
      2.64365505e+000, 2.20202699e+000, 5.54225147e-001, 1.78260107e+002,   &
      7.61376625e-001, 7.67218745e-002, 6.02946891e-003, 1.55143296e-002,   &
      9.91630530e-002, 1.76175995e-001, 9.56782020e-001, 1.54330682e+000]
   case(41)
      fact(:) = [ &
      6.59532875e-001, 8.66145490e-002, 1.84545854e+000, 5.94774398e+000,   &
      1.25584405e+000, 6.40851475e-001, 1.22253422e-001, 1.66646050e-001,   &
      7.06638328e-001, 1.62853268e+000, 2.62381591e-003, 8.26257859e-003]
   case(42)
      fact(:) = [ &
      6.10160120e-001, 9.11628054e-002, 1.26544000e+000, 5.06776025e-001,   &
      1.97428762e+000, 5.89590381e+000, 6.48028962e-001, 1.46634108e+000,   &
      2.60380817e-003, 7.84336311e-003, 1.13887493e-001, 1.55114340e-001]
   case(43)
      fact(:) = [ &
      8.55189183e-001, 1.02962151e-001, 1.66219641e+000, 7.64907000e+000,   &
      1.45575475e+000, 1.01639987e+000, 1.05445664e-001, 1.42303338e-001,   &
      7.71657112e-001, 1.34659349e+000, 2.20992635e-003, 7.90358976e-003]
   case(44)
      fact(:) = [ &
      4.70847093e-001, 9.33029874e-002, 1.58180781e+000, 4.52831347e-001,   &
      2.02419818e+000, 7.11489023e+000, 1.97036257e-003, 7.56181595e-003,   &
      6.26912639e-001, 1.25399858e+000, 1.02641320e-001, 1.33786087e-001]
   case(45)
      fact(:) = [ &
      4.20051553e-001, 9.38882628e-002, 1.76266507e+000, 4.64441687e-001,   &
      2.02735641e+000, 8.19346046e+000, 1.45487176e-003, 7.82704517e-003,   &
      6.22809600e-001, 1.17194153e+000, 9.91529915e-002, 1.24532839e-001]
   case(46)
      fact(:) = [ &
      2.10475155e+000, 8.68606470e+000, 2.03884487e+000, 3.78924449e-001,   &
      1.82067264e-001, 1.42921634e-001, 9.52040948e-002, 1.17125900e-001,   &
      5.91445248e-001, 1.07843808e+000, 1.13328676e-003, 7.80252092e-003]
   case(47)
      fact(:) = [ &
      2.07981390e+000, 9.92540297e+000, 4.43170726e-001, 1.04920104e-001,   &
      1.96515215e+000, 6.40103839e-001, 5.96130591e-001, 8.89594790e-001,   &
      4.78016333e-001, 1.98509407e+000, 9.46458470e-002, 1.12744464e-001]
   case(48)
      fact(:) = [ &
      1.63657549e+000, 1.24540381e+001, 2.17927989e+000, 1.45134660e+000,   &
      7.71300690e-001, 1.26695757e-001, 6.64193880e-001, 7.77659202e-001,   &
      7.64563285e-001, 1.66075210e+000, 8.61126689e-002, 1.05728357e-001]
   case(49)
      fact(:) = [ &
      2.24820632e+000, 1.51913507e+000, 1.64706864e+000, 1.30113424e+001,   &
      7.88679265e-001, 1.06128184e-001, 8.12579069e-002, 9.94045620e-002,   &
      6.68280346e-001, 1.49742063e+000, 6.38467475e-001, 7.18422635e-001]
   case(50)
      fact(:) = [ &
      2.16644620e+000, 1.13174909e+001, 6.88691021e-001, 1.10131285e-001,   &
      1.92431751e+000, 6.74464853e-001, 5.65359888e-001, 7.33564610e-001,   &
      9.18683861e-001, 1.02310312e+001, 7.80542213e-002, 9.31104308e-002]
   case(51)
      fact(:) = [ &
      1.73662114e+000, 8.84334719e-001, 9.99871380e-001, 1.38462121e-001,   &
      2.13972409e+000, 1.19666432e+001, 5.60566526e-001, 6.72672880e-001,   &
      9.93772747e-001, 8.72330411e+000, 7.37374982e-002, 8.78577715e-002]
   case(52)
      fact(:) = [ &
      2.09383882e+000, 1.26856869e+001, 1.56940519e+000, 1.21236537e+000,   &
      1.30941993e+000, 1.66633292e-001, 6.98067804e-002, 8.30817576e-002,   &
      1.04969537e+000, 7.43147857e+000, 5.55594354e-001, 6.17487676e-001]
   case(53)
      fact(:) = [ &
      1.60186925e+000, 1.95031538e-001, 1.98510264e+000, 1.36976183e+001,   &
      1.48226200e+000, 1.80304795e+000, 5.53807199e-001, 5.67912340e-001,   &
      1.11728722e+000, 6.40879878e+000, 6.60720847e-002, 7.86615429e-002]
   case(54)
      fact(:) = [ &
      1.60015487e+000, 2.92913354e+000, 1.71644581e+000, 1.55882990e+001,   &
      1.84968351e+000, 2.22525983e-001, 6.23813648e-002, 7.45581223e-002,   &
      1.21387555e+000, 5.56013271e+000, 5.54051946e-001, 5.21994521e-001]
   case(55)
      fact(:) = [ &
      2.95236854e+000, 6.01461952e+000, 4.28105721e-001, 4.64151246e+001,   &
      1.89599233e+000, 1.80109756e-001, 5.48012938e-002, 7.12799633e-002,   &
      4.70838600e+000, 4.56702799e+001, 5.90356719e-001, 4.70236310e-001]
   case(56)
      fact(:) = [ &
      3.19434243e+000, 9.27352241e+000, 1.98289586e+000, 2.28741632e-001,   &
      1.55121052e-001, 3.82000231e-002, 6.73222354e-002, 7.30961745e-002,   &
      4.48474211e+000, 2.95703565e+001, 5.42674414e-001, 4.08647015e-001]
   case(57)
      fact(:) = [ &
      2.05036425e+000, 2.20348417e-001, 1.42114311e-001, 3.96438056e-002,   &
      3.23538151e+000, 9.56979169e+000, 6.34683429e-002, 6.92443091e-002,   &
      3.97960586e+000, 2.53178406e+001, 5.20116711e-001, 3.83614098e-001]
   case(58)
      fact(:) = [ &
      3.22990759e+000, 9.94660135e+000, 1.57618307e-001, 4.15378676e-002,   &
      2.13477838e+000, 2.40480572e-001, 5.01907609e-001, 3.66252019e-001,   &
      3.80889010e+000, 2.43275968e+001, 5.96625028e-002, 6.59653503e-002]
   case(59)
      fact(:) = [ &
      1.58189324e-001, 3.91309056e-002, 3.18141995e+000, 1.04139545e+001,   &
      2.27622140e+000, 2.81671757e-001, 3.97705472e+000, 2.61872978e+001,   &
      5.58448277e-002, 6.30921695e-002, 4.85207954e-001, 3.54234369e-001]
   case(60)
      fact(:) = [ &
      1.81379417e-001, 4.37324793e-002, 3.17616396e+000, 1.07842572e+001,   &
      2.35221519e+000, 3.05571833e-001, 3.83125763e+000, 2.54745408e+001,   &
      5.25889976e-002, 6.02676073e-002, 4.70090742e-001, 3.39017003e-001]
   case(61)
      fact(:) = [ &
      1.92986811e-001, 4.37785970e-002, 2.43756023e+000, 3.29336996e-001,   &
      3.17248504e+000, 1.11259996e+001, 3.58105414e+000, 2.46709586e+001,   &
      4.56529394e-001, 3.24990282e-001, 4.94812177e-002, 5.76553100e-002]
   case(62)
      fact(:) = [ &
      2.12002595e-001, 4.57703608e-002, 3.16891754e+000, 1.14536599e+001,   &
      2.51503494e+000, 3.55561054e-001, 4.44080845e-001, 3.11953363e-001,   &
      3.36742101e+000, 2.40291435e+001, 4.65652543e-002, 5.52266819e-002]
   case(63)
      fact(:) = [ &
      2.59355002e+000, 3.82452612e-001, 3.16557522e+000, 1.17675155e+001,   &
      2.29402652e-001, 4.76642249e-002, 4.32257780e-001, 2.99719833e-001,   &
      3.17261920e+000, 2.34462738e+001, 4.37958317e-002, 5.29440680e-002]
   case(64)
      fact(:) = [ &
      3.19144939e+000, 1.20224655e+001, 2.55766431e+000, 4.08338876e-001,   &
      3.32681934e-001, 5.85819814e-002, 4.14243130e-002, 5.06771477e-002,   &
      2.61036728e+000, 1.99344244e+001, 4.20526863e-001, 2.85686240e-001]
   case(65)
      fact(:) = [ &
      2.59407462e-001, 5.04689354e-002, 3.16177855e+000, 1.23140183e+001,   &
      2.75095751e+000, 4.38337626e-001, 2.79247686e+000, 2.23797309e+001,   &
      3.85931001e-002, 4.87920992e-002, 4.10881708e-001, 2.77622892e-001]
   case(66)
      fact(:) = [ &
      3.16055396e+000, 1.25470414e+001, 2.82751709e+000, 4.67899094e-001,   &
      2.75140255e-001, 5.23226982e-002, 4.00967160e-001, 2.67614884e-001,   &
      2.63110834e+000, 2.19498166e+001, 3.61333817e-002, 4.68871497e-002]
   case(67)
      fact(:) = [ &
      2.88642467e-001, 5.40507687e-002, 2.90567296e+000, 4.97581077e-001,   &
      3.15960159e+000, 1.27599505e+001, 3.91280259e-001, 2.58151831e-001,   &
      2.48596038e+000, 2.15400972e+001, 3.37664478e-002, 4.50664323e-002]
   case(68)
      fact(:) = [ &
      3.15573213e+000, 1.29729009e+001, 3.11519560e-001, 5.81399387e-002,   &
      2.97722406e+000, 5.31213394e-001, 3.81563854e-001, 2.49195776e-001,   &
      2.40247532e+000, 2.13627616e+001, 3.15224214e-002, 4.33253257e-002]
   case(69)
      fact(:) = [ &
      3.15591970e+000, 1.31232407e+001, 3.22544710e-001, 5.97223323e-002,   &
      3.05569053e+000, 5.61876773e-001, 2.92845100e-002, 4.16534255e-002,   &
      3.72487205e-001, 2.40821967e-001, 2.27833695e+000, 2.10034185e+001]
   case(70)
      fact(:) = [ &
      3.10794704e+000, 6.06347847e-001, 3.14091221e+000, 1.33705269e+001,   &
      3.75660454e-001, 7.29814740e-002, 3.61901097e-001, 2.32652051e-001,   &
      2.45409082e+000, 2.12695209e+001, 2.72383990e-002, 3.99969597e-002]
   case(71)
      fact(:) = [ &
      3.11446863e+000, 1.38968881e+001, 5.39634353e-001, 8.91708508e-002,   &
      3.06460915e+000, 6.79919563e-001, 2.58563745e-002, 3.82808522e-002,   &
      2.13983556e+000, 1.80078788e+001, 3.47788231e-001, 2.22706591e-001]
   case(72)
      fact(:) = [ &
      3.01166899e+000, 7.10401889e-001, 3.16284788e+000, 1.38262192e+001,   &
      6.33421771e-001, 9.48486572e-002, 3.41417198e-001, 2.14129678e-001,   &
      1.53566013e+000, 1.55298698e+001, 2.40723773e-002, 3.67833690e-002]
   case(73)
      fact(:) = [ &
      3.20236821e+000, 1.38446369e+001, 8.30098413e-001, 1.18381581e-001,   &
      2.86552297e+000, 7.66369118e-001, 2.24813887e-002, 3.52934622e-002,   &
      1.40165263e+000, 1.46148877e+001, 3.33740596e-001, 2.05704486e-001]
   case(74)
      fact(:) = [ &
      9.24906855e-001, 1.28663377e-001, 2.75554557e+000, 7.65826479e-001,   &
      3.30440060e+000, 1.34471170e+001, 3.29973862e-001, 1.98218895e-001,   &
      1.09916444e+000, 1.35087534e+001, 2.06498883e-002, 3.38918459e-002]
   case(75)
      fact(:) = [ &
      1.96952105e+000, 4.98830620e+001, 1.21726619e+000, 1.33243809e-001,   &
      4.10391685e+000, 1.84396916e+000, 2.90791978e-002, 2.84192813e-002,   &
      2.30696669e-001, 1.90968784e-001, 6.08840299e-001, 1.37090356e+000]
   case(76)
      fact(:) = [ &
      2.06385867e+000, 4.05671697e+001, 1.29603406e+000, 1.46559047e-001,   &
      3.96920673e+000, 1.82561596e+000, 2.69835487e-002, 2.84172045e-002,   &
      2.31083999e-001, 1.79765184e-001, 6.30466774e-001, 1.38911543e+000]
   case(77)
      fact(:) = [ &
      2.21522726e+000, 3.24464090e+001, 1.37573155e+000, 1.60920048e-001,   &
      3.78244405e+000, 1.78756553e+000, 2.44643240e-002, 2.82909938e-002,   &
      2.36932016e-001, 1.70692368e-001, 6.48471412e-001, 1.37928390e+000]
   case(78)
      fact(:) = [ &
      9.84697940e-001, 1.60910839e-001, 2.73987079e+000, 7.18971667e-001,   &
      3.61696715e+000, 1.29281016e+001, 3.02885602e-001, 1.70134854e-001,   &
      2.78370726e-001, 1.49862703e+000, 1.52124129e-002, 2.83510822e-002]
   case(79)
      fact(:) = [ &
      9.61263398e-001, 1.70932277e-001, 3.69581030e+000, 1.29335319e+001,   &
      2.77567491e+000, 6.89997070e-001, 2.95414176e-001, 1.63525510e-001,   &
      3.11475743e-001, 1.39200901e+000, 1.43237267e-002, 2.71265337e-002]
   case(80)
      fact(:) = [ &
      1.29200491e+000, 1.83432865e-001, 2.75161478e+000, 9.42368371e-001,   &
      3.49387949e+000, 1.46235654e+001, 2.77304636e-001, 1.55110144e-001,   &
      4.30232810e-001, 1.28871670e+000, 1.48294351e-002, 2.61903834e-002]
   case(81)
      fact(:) = [ &
      3.75964730e+000, 1.35041513e+001, 3.21195904e+000, 6.66330993e-001,   &
      6.47767825e-001, 9.22518234e-002, 2.76123274e-001, 1.50312897e-001,   &
      3.18838810e-001, 1.12565588e+000, 1.31668419e-002, 2.48879842e-002]
   case(82)
      fact(:) = [ &
      1.00795975e+000, 1.17268427e-001, 3.09796153e+000, 8.80453235e-001,   &
      3.61296864e+000, 1.47325812e+001, 2.62401476e-001, 1.43491014e-001,   &
      4.05621995e-001, 1.04103506e+000, 1.31812509e-002, 2.39575415e-002]
   case(83)
      fact(:) = [ &
      1.59826875e+000, 1.56897471e-001, 4.38233925e+000, 2.47094692e+000,   &
      2.06074719e+000, 5.72438972e+001, 1.94426023e-001, 1.32979109e-001,   &
      8.22704978e-001, 9.56532528e-001, 2.33226953e-002, 2.23038435e-002]
   case(84)
      fact(:) = [ &
      1.71463223e+000, 9.79262841e+001, 2.14115960e+000, 2.10193717e-001,   &
      4.37512413e+000, 3.66948812e+000, 2.16216680e-002, 1.98456144e-002,   &
      1.97843837e-001, 1.33758807e-001, 6.52047920e-001, 7.80432104e-001]
   case(85)
      fact(:) = [ &
      1.48047794e+000, 1.25943919e+002, 2.09174630e+000, 1.83803008e-001,   &
      4.75246033e+000, 4.19890596e+000, 1.85643958e-002, 1.81383503e-002,   &
      2.05859375e-001, 1.33035404e-001, 7.13540948e-001, 7.03031938e-001]
   case(86)
      fact(:) = [ &
      6.30022295e-001, 1.40909762e-001, 3.80962881e+000, 3.08515540e+001,   &
      3.89756067e+000, 6.51559763e-001, 2.40755100e-001, 1.08899672e-001,   &
      2.62868577e+000, 6.42383261e+000, 3.14285931e-002, 2.42346699e-002]
   case(87)
      fact(:) = [ &
      5.23288135e+000, 8.60599536e+000, 2.48604205e+000, 3.04543982e-001,   &
      3.23431354e-001, 3.87759096e-002, 2.55403596e-001, 1.28717724e-001,   &
      5.53607228e-001, 5.36977452e-001, 5.75278889e-003, 1.29417790e-002]
   case(88)
      fact(:) = [ &
      1.44192685e+000, 1.18740873e-001, 3.55291725e+000, 1.01739750e+000,   &
      3.91259586e+000, 6.31814783e+001, 2.16173519e-001, 9.55806441e-002,   &
      3.94191605e+000, 3.50602732e+001, 4.60422605e-002, 2.20850385e-002]
   case(89)
      fact(:) = [ &
      1.45864127e+000, 1.07760494e-001, 4.18945405e+000, 8.89090649e+001,   &
      3.65866182e+000, 1.05088931e+000, 2.08479229e-001, 9.09335557e-002,   &
      3.16528117e+000, 3.13297788e+001, 5.23892556e-002, 2.08807697e-002]
   case(90)
      fact(:) = [ &
      1.19014064e+000, 7.73468729e-002, 2.55380607e+000, 6.59693681e-001,   &
      4.68110181e+000, 1.28013896e+001, 2.26121303e-001, 1.08632194e-001,   &
      3.58250545e-001, 4.56765664e-001, 7.82263950e-003, 1.62623474e-002]
   case(91)
      fact(:) = [ &
      4.68537504e+000, 1.44503632e+001, 2.98413708e+000, 5.56438592e-001,   &
      8.91988061e-001, 6.69512914e-002, 2.24825384e-001, 1.03235396e-001,   &
      3.04444846e-001, 4.27255647e-001, 9.48162708e-003, 1.77730611e-002]
   case(92)
      fact(:) = [ &
      4.63343606e+000, 1.63377267e+001, 3.18157056e+000, 5.69517868e-001,   &
      8.76455075e-001, 6.88860012e-002, 2.21685477e-001, 9.84254550e-002,   &
      2.72917100e-001, 4.09470917e-001, 1.11737298e-002, 1.86215410e-002]
   case(93)
      fact(:) = [ &
      4.56773888e+000, 1.90992795e+001, 3.40325179e+000, 5.90099634e-001,   &
      8.61841923e-001, 7.03204851e-002, 2.19728870e-001, 9.36334280e-002,   &
      2.38176903e-001, 3.93554882e-001, 1.38306499e-002, 1.94437286e-002]
   case(94)
      fact(:) = [ &
      5.45671123e+000, 1.01892720e+001, 1.11687906e-001, 3.98131313e-002,   &
      3.30260343e+000, 3.14622212e-001, 1.84568319e-001, 1.04220860e-001,   &
      4.93644263e-001, 4.63080540e-001, 3.57484743e+000, 2.19369542e+001]
   case(95)
      fact(:) = [ &
      5.38321999e+000, 1.07289857e+001, 1.23343236e-001, 4.15137806e-002,   &
      3.46469090e+000, 3.39326208e-001, 1.75437132e-001, 9.98932346e-002,   &
      3.39800073e+000, 2.11601535e+001, 4.69459519e-001, 4.51996970e-001]
   case(96)
      fact(:) = [ &
      5.38402377e+000, 1.11211419e+001, 3.49861264e+000, 3.56750210e-001,   &
      1.88039547e-001, 5.39853583e-002, 1.69143137e-001, 9.60082633e-002,   &
      3.19595016e+000, 1.80694389e+001, 4.64393059e-001, 4.36318197e-001]
   case(97)
      fact(:) = [ &
      3.66090688e+000, 3.84420906e-001, 2.03054678e-001, 5.48547131e-002,   &
      5.30697515e+000, 1.17150262e+001, 1.60934046e-001, 9.21020329e-002,   &
      3.04808401e+000, 1.73525367e+001, 4.43610295e-001, 4.27132359e-001]
   case(98)
      fact(:) = [ &
      3.94150390e+000, 4.18246722e-001, 5.16915345e+000, 1.25201788e+001,   &
      1.61941074e-001, 4.81540117e-002, 4.15299561e-001, 4.24913856e-001,   &
      2.91761325e+000, 1.90899693e+001, 1.51474927e-001, 8.81568925e-002]
   case(99)
      fact(:) = [ &
      4.09780623e+000, 4.46021145e-001, 5.10079393e+000, 1.31768613e+001,   &
      1.74617289e-001, 5.02742829e-002, 2.76774658e+000, 1.84815393e+001,   &
      1.44496639e-001, 8.46232592e-002, 4.02772109e-001, 4.17640100e-001]
   case(100)
      fact(:) = [ &
      4.24934820e+000, 4.75263933e-001, 5.03556594e+000, 1.38570834e+001,   &
      1.88920613e-001, 5.26975158e-002, 3.94356058e-001, 4.11193751e-001,   &
      2.61213100e+000, 1.78537905e+001, 1.38001927e-001, 8.12774434e-002]
   case(101)
      fact(:) = [ &
      2.00942931e-001, 5.48366518e-002, 4.40119869e+000, 5.04248434e-001,   &
      4.97250102e+000, 1.45721366e+001, 2.47530599e+000, 1.72978308e+001,   &
      3.86883197e-001, 4.05043898e-001, 1.31936095e-001, 7.80821071e-002]
   case(102)
      fact(:) = [ &
      2.16052899e-001, 5.83584058e-002, 4.91106799e+000, 1.53264212e+001,   &
      4.54862870e+000, 5.34434760e-001, 2.36114249e+000, 1.68164803e+001,   &
      1.26277292e-001, 7.50304633e-002, 3.81364501e-001, 3.99305852e-001]
   case(103)
      fact(:) = [ &
      4.86738014e+000, 1.60320520e+001, 3.19974401e-001, 6.70871138e-002,   &
      4.58872425e+000, 5.77039373e-001, 1.21482448e-001, 7.22275899e-002,   &
      2.31639872e+000, 1.41279737e+001, 3.79258137e-001, 3.89973484e-001]
   case default
      ier = 1
      fact(:) = 0
   end select

   end subroutine get_kirkland_params

!----------------------------------------------------------------------------------------------------------------

   subroutine get_lobato_params(zval,fact,ier)
!
!  I. Lobato and D. Van Dyck, Acta Cryst. (2014). A70, 636-649
!  An accurate parameterization for scattering factors,
!  electron densities and electrostatic potentials for
!  neutral atoms that obey all physical constraints
!
   use type_constants, only: DP
   integer, intent(in)                        :: zval
   real(DP), dimension(NPARLOBA), intent(out) :: fact
   integer, intent(out)                       :: ier
!
   ier = 0
!
!  Order of parameters:
!           a1    a2    a3    a4    a5
!           b1    b2    b3    b4    b5
   select case(zval)
   case(1)
      fact(:) = [ &
      6.473848488352918e-03, -4.901925767802290e-01, 5.732841603908765e-01, -3.794033014839905e-01, 5.544264747740791e-01,   &
      2.785198853791489e+00, 2.776204283306448e+00, 2.775385910506251e+00, 2.767593028672588e+00, 2.765118976429275e+00]
   case(2)
      fact(:) = [ &
      3.057451160998355e+00, -6.200447791273253e+01, 6.400555370846145e+01, -5.001325785427806e+00, 1.517988287005264e-01,   &
      1.089672487260788e+00, 9.398387981431211e-01, 9.252890343862655e-01, 8.229474987086506e-01, 5.773931106754022e-01]
   case(3)
      fact(:) = [ &
      3.926222728861479e+00, -4.548619626399980e+00, 2.193353128786585e+00, 6.994512650339657e-02, 2.098642248519376e-03,   &
      8.142760135172804e+00, 4.989410770078558e+00, 4.144289992394109e+00, 4.019223150656802e-01, 1.564790347198236e-01]
   case(4)
      fact(:) = [ &
      3.398249705570541e+00, -1.908668860956967e+00, 3.907021175392274e-02, -1.116310102107145e-02, 9.462044653575231e-03,   &
      4.442701786224095e+00, 3.324515425264230e+00, 1.897728803482149e-01, 8.719186146446036e-02, 8.278090600413406e-02]
   case(5)
      fact(:) = [ &
      1.472792486393293e+00, -4.019330421993871e-01, 3.059989569826894e-01, 1.961442171731680e-02, 9.771771060882025e-04,   &
      3.749740482818191e+00, 5.880665361396738e-01, 5.156396131030110e-01, 1.213775700806037e-01, 6.809824121603139e-02]
   case(6)
      fact(:) = [ &
      1.244660886213433e+02, -2.203528570789638e+02, 1.952353522804791e+02, -9.810793612697997e+01, 1.420230412136232e-02,   &
      2.421208492560056e+00, 2.305379437524258e+00, 2.048519321065642e+00, 1.933525529175474e+00, 7.689768184783397e-02]
   case(7)
      fact(:) = [ &
      5.813271507025561e+01, -1.475424090878127e+02, 1.301430656496395e+02, -3.961956740841543e+01, 1.059577633314809e-02,   &
      1.700448564134711e+00, 1.559038526017404e+00, 1.415768274731469e+00, 1.278418182054558e+00, 5.655877984748055e-02]
   case(8)
      fact(:) = [ &
      2.994740452423624e+01, -7.761012662552783e+01, 9.988177646231442e+01, -5.121270055056731e+01, 8.196189544460320e-03,   &
      1.302839878800107e+00, 1.157941052583095e+00, 1.009885493380251e+00, 9.433279714332660e-01, 4.331976113218256e-02]
   case(9)
      fact(:) = [ &
      9.489848945035248e-01, -3.013339230435549e+01, 5.279650781273386e+01, -2.270627037952724e+01, 6.569976645314410e-03,   &
      1.458829331986459e+00, 6.887799931876800e-01, 6.542398693466957e-01, 6.148361308119943e-01, 3.428374194950112e-02]
   case(10)
      fact(:) = [ &
      5.827411922209074e-01, 3.706765618410549e-01, -5.467449673508092e-01, 4.140526824802081e-01, 5.199030808639931e-03,   &
      1.281185731438772e+00, 4.445208971704776e-01, 1.986508755104810e-01, 1.854772466562765e-01, 2.757383820338858e-02]
   case(11)
      fact(:) = [ &
      2.367006039467926e+01, -2.185317861597425e+01, 5.924994481089464e-01, -2.446522903102440e-02, 4.839502217065358e-03,   &
      8.451487735146031e+00, 8.040966004742982e+00, 6.249960005263150e-01, 1.324503949472964e-01, 2.339943620498786e-02]
   case(12)
      fact(:) = [ &
      4.855010476871495e+00, -2.662209064768437e+00, 4.780012360851080e-01, -7.023070646920648e-02, 3.989058281040198e-03,   &
      5.946392738424565e+00, 4.171303125206979e+00, 3.982698081503743e-01, 1.618861858374742e-01, 1.953450563631035e-02]
   case(13)
      fact(:) = [ &
      2.834095616075075e+00, -4.280041333782610e+00, 4.421916805483114e+00, -3.457744718964006e-02, 3.523859414060799e-03,   &
      6.662350239805332e+00, 5.512947222240214e-01, 5.093289634459738e-01, 1.117848374253312e-01, 1.676023518052574e-02]
   case(14)
      fact(:) = [ &
      2.871891426116124e+00, -2.061735011951735e+00, 2.171140242044787e+00, -6.630736330588019e-02, 3.010707096705137e-03,   &
      5.084871036429896e+00, 4.291781853051262e-01, 3.664854341921622e-01, 1.197106112969034e-01, 1.439945361283975e-02]
   case(15)
      fact(:) = [ &
      2.791518400231514e+00, -4.365068378238221e+00, 4.435584555166990e+00, -8.096357733994739e-02, 2.679000179664404e-03,   &
      3.900659618654661e+00, 3.298259683771500e-01, 3.060899565058886e-01, 1.080832325459728e-01, 1.258944953311868e-02]
   case(16)
      fact(:) = [ &
      2.679714156101995e+00, -4.742528222307552e-01, 5.148359489896877e-01, -9.583600249907225e-02, 2.488719638189352e-03,   &
      3.068891211999711e+00, 3.782167021858090e-01, 1.887218119025488e-01, 9.233705900314941e-02, 1.119208772411441e-02]
   case(17)
      fact(:) = [ &
      2.566248399800203e+00, -3.388763508285917e-01, 1.145845587555150e+00, -9.231093165470796e-01, 2.291680020410422e-03,   &
      2.415949203656124e+00, 4.214142393102160e-01, 1.095924049758303e-01, 9.909554582267530e-02, 9.996659489275210e-03]
   case(18)
      fact(:) = [ &
      2.459817464140685e+00, -3.641981770769951e-01, 2.505844772224745e-01, -5.774370295443458e-02, 2.301438668285837e-03,   &
      1.940046319888566e+00, 3.992410678843988e-01, 1.174724062744122e-01, 5.678037260236218e-02, 9.155798329207080e-03]
   case(19)
      fact(:) = [ &
      5.811078786014548e+00, -5.025370965394226e+01, 4.886094120598425e+01, 7.406285920483829e-02, 7.278027386262219e-04,   &
      1.266914833990368e+01, 3.956410396981665e+00, 3.683850595771545e+00, 1.074585175695628e-01, 6.655767893915011e-03]
   case(20)
      fact(:) = [ &
      2.117811615241595e+01, -3.390438243174689e+02, 3.227569585232967e+02, 6.500776738969967e-02, 6.558743665785935e-04,   &
      6.396086194317362e+00, 3.740247138917495e+00, 3.648884499226053e+00, 9.450906345146735e-02, 5.985206198837586e-03]
   case(21)
      fact(:) = [ &
      1.260351865721486e+01, -2.768753820537026e+02, 2.688716039073428e+02, 5.568241788974580e-02, 5.770712551453996e-04,   &
      6.156256153638529e+00, 3.088735542666793e+00, 3.027276632985483e+00, 8.188747483752157e-02, 5.382898320507976e-03]
   case(22)
      fact(:) = [ &
      8.575957752381299e+00, -2.103315634653049e+02, 2.060971726015414e+02, 4.777739489772619e-02, 5.057164844803206e-04,   &
      6.007806688755818e+00, 2.602858567452130e+00, 2.553523450511056e+00, 7.114294840241539e-02, 4.856284393837261e-03]
   case(23)
      fact(:) = [ &
      6.527684332347890e+00, -2.004305768291727e+02, 1.980150538899940e+02, 4.139115180640056e-02, 4.474550243298565e-04,   &
      5.835524793524481e+00, 2.232559523811276e+00, 2.197860185942290e+00, 6.239738768285473e-02, 4.403836490799580e-03]
   case(24)
      fact(:) = [ &
      3.028317848436913e+00, -9.553939330814323e+01, 9.617615623521981e+01, 3.597773159579877e-02, 3.914928907184223e-04,   &
      8.359115043146318e+00, 1.802637902641032e+00, 1.775094889570472e+00, 5.481444121431137e-02, 3.998289689160348e-03]
   case(25)
      fact(:) = [ &
      4.374175506331227e+00, -1.609255109187795e+02, 1.602733080603223e+02, 3.123038610491624e-02, 3.469660210209380e-04,   &
      5.510317055049282e+00, 1.687982164024339e+00, 1.666140477777024e+00, 4.833703903212770e-02, 3.647469600530682e-03]
   case(26)
      fact(:) = [ &
      3.798100908368596e+00, -9.168935493816872e+01, 9.144542521554298e+01, 2.727543440276047e-02, 3.033798543837710e-04,   &
      5.317126458994331e+00, 1.497130948847481e+00, 1.468092418044105e+00, 4.272478501289549e-02, 3.327918552318741e-03]
   case(27)
      fact(:) = [ &
      3.330378744675443e+00, -7.700175964729671e+01, 7.707252217905169e+01, 2.399046690405699e-02, 2.682566655239420e-04,   &
      5.181359645795432e+00, 1.329151222651394e+00, 1.302849289632289e+00, 3.806454868263710e-02, 3.050100079915708e-03]
   case(28)
      fact(:) = [ &
      2.969080787252864e+00, -7.574770691290402e+01, 7.603982876253073e+01, 2.101621136929670e-02, 2.311517511351530e-04,   &
      5.041809491093800e+00, 1.182755079216293e+00, 1.162165458466299e+00, 3.374790878830353e-02, 2.786808620707405e-03]
   case(29)
      fact(:) = [ &
      1.752071452121456e+00, -4.304105234921245e+01, 4.407059155435739e+01, 1.868761540881447e-02, 2.017273247916241e-04,   &
      6.187504979861871e+00, 1.002662636289766e+00, 9.853843113530303e-01, 3.029847039161176e-02, 2.558555987488791e-03]
   case(30)
      fact(:) = [ &
      2.466371104994591e+00, -6.146785413325375e+01, 6.201769452374813e+01, 1.641601739314162e-02, 1.724871175953809e-04,   &
      4.910280784938159e+00, 9.678985203229922e-01, 9.512838347753555e-01, 2.696009676675663e-02, 2.341096110462537e-03]
   case(31)
      fact(:) = [ &
      2.760102031084283e+00, -3.444526142074679e+01, 3.522622672440163e+01, 1.320671969994199e-02, 1.259455609282457e-04,   &
      6.101282245376627e+00, 7.651433135534649e-01, 7.513286233828597e-01, 2.248796343172512e-02, 2.067373742787123e-03]
   case(32)
      fact(:) = [ &
      3.182416352600000e+00, -5.245140378111667e+01, 5.296908271622185e+01, 1.140961685918648e-02, 9.509543581450579e-05,   &
      5.017190408609147e+00, 7.123957644377982e-01, 7.022801925281940e-01, 1.967472956940154e-02, 1.841466144940115e-03]
   case(33)
      fact(:) = [ &
      3.456429691196040e+00, -3.331760444317212e+01, 3.357121938553322e+01, 9.790022956403421e-03, 6.534348622651725e-05,   &
      4.013580160329452e+00, 6.623557780506291e-01, 6.457719410560738e-01, 1.709193532310115e-02, 1.603016028394201e-03]
   case(34)
      fact(:) = [ &
      3.649050478019264e+00, -4.368516622212386e+01, 4.369202886011548e+01, 8.449022841991444e-03, 3.786114741100382e-05,   &
      3.250432671125934e+00, 6.096662016502890e-01, 5.969713008021232e-01, 1.485545127403623e-02, 1.336255873565630e-03]
   case(35)
      fact(:) = [ &
      3.838463122242895e+00, -5.227234710112339e+01, 5.198612794956596e+01, 7.339559893380448e-03, 1.646942079513399e-05,   &
      2.611894705732323e+00, 5.661950628747182e-01, 5.552793266998739e-01, 1.297464694324407e-02, 1.029865368558757e-03]
   case(36)
      fact(:) = [ &
      4.025410303108274e+00, -4.630423320789298e+01, 4.572136819041012e+01, 6.353379596253157e-03, 1.334778452557430e-06,   &
      2.136483814437400e+00, 5.265911664541312e-01, 5.141367844645776e-01, 1.128072422420068e-02, 4.880898579409704e-04]
   case(37)
      fact(:) = [ &
      3.389753515953940e+00, 2.143483486791724e+00, 3.543226035109782e-01, 3.740093400856642e-03, 3.003425023422297e-07,   &
      2.057448143681711e+01, 1.910799452185164e+00, 1.974105893966039e-01, 8.134594653439889e-03, 2.926857910569511e-04]
   case(38)
      fact(:) = [ &
      4.770925092998260e+00, 1.475978501552834e+00, 3.044513555441887e-01, 3.594749819167283e-03, 3.000855492282790e-07,   &
      1.336688813304128e+01, 1.337383795771967e+00, 1.775323694377828e-01, 7.791050330018268e-03, 2.822551396485372e-04]
   case(39)
      fact(:) = [ &
      4.607210198752340e+00, 1.428018510398403e+00, 2.955810455777538e-01, 3.389978408528951e-03, 2.668629749690224e-07,   &
      1.086869055571519e+01, 1.311374558731241e+00, 1.680228707847112e-01, 7.359645453504783e-03, 2.623432810213873e-04]
   case(40)
      fact(:) = [ &
      4.311754534068716e+00, 1.493315780393395e+00, 2.812360501288841e-01, 3.093377384557471e-03, 2.580244485125306e-07,   &
      9.458965805174902e+00, 1.330636228452456e+00, 1.565068714444536e-01, 6.824105238117352e-03, 2.498188791107023e-04]
   case(41)
      fact(:) = [ &
      3.111790391134954e+00, 2.202590609458031e+00, 2.703307749100209e-01, 2.687991947510981e-03, 2.325494833477792e-07,   &
      1.069031413430237e+01, 1.653163561589331e+00, 1.451151857189699e-01, 6.139563515828505e-03, 2.322402359376725e-04]
   case(42)
      fact(:) = [ &
      2.831059684320536e+00, 2.348581374896745e+00, 2.451058884296964e-01, 2.352821082185159e-03, 2.312708383438592e-07,   &
      1.043571957589507e+01, 1.604828686745972e+00, 1.316969347746069e-01, 5.549779013833459e-03, 2.227474637648977e-04]
   case(43)
      fact(:) = [ &
      2.571798593233526e+00, 2.456337419572030e+00, 2.206584408813715e-01, 2.055314180124382e-03, 2.321329477936922e-07,   &
      1.016431171313775e+01, 1.534419192445505e+00, 1.191386197541104e-01, 5.018532408829888e-03, 2.145903791208612e-04]
   case(44)
      fact(:) = [ &
      2.332300303539466e+00, 2.535780254890496e+00, 1.982080421360260e-01, 1.761201304600535e-03, 1.981294115460245e-07,   &
      9.921674601599595e+00, 1.455856688087881e+00, 1.076821878733493e-01, 4.472435456549673e-03, 1.965747162324994e-04]
   case(45)
      fact(:) = [ &
      2.113525348594700e+00, 2.586363161550138e+00, 1.770639138636865e-01, 1.497370510030569e-03, 2.054814455556224e-07,   &
      9.659137258597623e+00, 1.371066569343296e+00, 9.703530275846488e-02, 3.971285090887921e-03, 1.913551907377983e-04]
   case(46)
      fact(:) = [ &
      6.421597961826873e-01, 2.979148144263289e+00, 1.681544260042708e-01, 1.337442138784078e-03, 1.914109682468616e-07,   &
      5.974797502634061e+00, 1.433594325412777e+00, 9.098684011726770e-02, 3.624101371161622e-03, 1.806889144160450e-04]
   case(47)
      fact(:) = [ &
      1.553172166803896e+00, 2.639303646999888e+00, 1.420154869567882e-01, 1.008504600997609e-03, 1.946384299730244e-07,   &
      8.156202357589566e+00, 1.216008874818015e+00, 7.900988649158734e-02, 2.965479013639490e-03, 1.745930959191462e-04]
   case(48)
      fact(:) = [ &
      6.153078519928602e+01, -7.860167412015821e+01, 2.155012926027705e+01, 1.376850156641916e-01, 3.246449309465210e-04,   &
      3.114681025332473e+00, 2.760169833885745e+00, 1.935513123247224e+00, 7.224683472602932e-02, 1.170016296246845e-03]
   case(49)
      fact(:) = [ &
      4.222321779015246e+00, -2.641213183532026e+01, 2.728528527087469e+01, 1.216179018841380e-01, 3.068835463715258e-04,   &
      6.072655104032275e+00, 1.645501789593879e+00, 1.522570749395068e+00, 6.565079146952256e-02, 1.120938514730356e-03]
   case(50)
      fact(:) = [ &
      5.142220746420537e+00, -2.549454137625767e+01, 2.574144875486019e+01, 1.117782275921996e-01, 2.936473847377427e-04,   &
      5.272726364744845e+00, 1.531949592091483e+00, 1.402575250400872e+00, 6.116853872630869e-02, 1.075608153945223e-03]
   case(51)
      fact(:) = [ &
      6.241640318318837e+00, -9.338687244195522e+01, 9.263328758298843e+01, 1.034126922212987e-01, 2.818484268943907e-04,   &
      4.269841080826875e+00, 1.394077461402502e+00, 1.355668549104345e+00, 5.726679496521998e-02, 1.033079542820674e-03]
   case(52)
      fact(:) = [ &
      7.377433018134030e+00, -1.260251069316281e+02, 1.241284050040956e+02, 9.599783718185698e-02, 2.710722163988820e-04,   &
      3.469177577838978e+00, 1.297598108867366e+00, 1.267710676460660e+00, 5.377717501794231e-02, 9.930845770291768e-04]
   case(53)
      fact(:) = [ &
      9.644006662721232e+00, -1.229244353501125e+02, 1.186825648165673e+02, 8.950259470255390e-02, 2.612761214904750e-04,   &
      2.726455453665441e+00, 1.237234258508660e+00, 1.200620368722498e+00, 5.066786822851999e-02, 9.554378833834973e-04]
   case(54)
      fact(:) = [ &
      1.554517496748606e+01, -1.182410278567446e+02, 1.080095249631970e+02, 8.362593419922217e-02, 2.519918624290739e-04,   &
      2.106373408654927e+00, 1.208603761295122e+00, 1.153952705672140e+00, 4.781893912748243e-02, 9.198862575926131e-04]
   case(55)
      fact(:) = [ &
      4.287087391816923e+00, 3.232506654229650e+00, 6.740295335617063e-01, 6.189080833876976e-02, 2.356120529521900e-04,   &
      2.265878707985415e+01, 2.237973864700642e+00, 3.689955686643163e-01, 4.022665752984844e-02, 8.837618908780351e-04]
   case(56)
      fact(:) = [ &
      6.244751873904615e+00, 2.351722714163885e+00, 4.742793732222520e-01, 6.381138742868499e-02, 2.346512805627924e-04,   &
      1.514313541909856e+01, 1.453790006048140e+00, 3.208356463878018e-01, 4.043545320946998e-02, 8.540811312800327e-04]
   case(57)
      fact(:) = [ &
      6.097881795995097e+00, 2.194951647366750e+00, 5.481727919600226e-01, 6.166695732226626e-02, 2.268073558647142e-04,   &
      1.242885443158548e+01, 1.505359923520235e+00, 3.337380397171243e-01, 3.877445534999988e-02, 8.240498840648670e-04]
   case(58)
      fact(:) = [ &
      5.795268796472405e+00, 2.370226641078433e+00, 4.713987569011149e-01, 5.743682605878668e-02, 2.189794892596593e-04,   &
      1.428010550582560e+01, 1.359690157191345e+00, 3.020173496635141e-01, 3.664367981470077e-02, 7.954345265183711e-04]
   case(59)
      fact(:) = [ &
      5.604062553775258e+00, 2.357962595618129e+00, 4.760010985728824e-01, 5.441233743152471e-02, 2.114146022051498e-04,   &
      1.395174902305747e+01, 1.312397549174392e+00, 2.949337011929565e-01, 3.486015424623241e-02, 7.682616826618097e-04]
   case(60)
      fact(:) = [ &
      5.429083919697703e+00, 2.336873253608803e+00, 4.833735541048819e-01, 5.146549645574790e-02, 2.037761328637611e-04,   &
      1.365036494276600e+01, 1.267598413903199e+00, 2.886106815769299e-01, 3.312918074466093e-02, 7.423484838012663e-04]
   case(61)
      fact(:) = [ &
      5.267744450894446e+00, 2.308558133263050e+00, 4.932654790260128e-01, 4.863562797416966e-02, 1.963088423206752e-04,   &
      1.336020968273946e+01, 1.225858565869417e+00, 2.829196321278669e-01, 3.147353971271379e-02, 7.176580250696152e-04]
   case(62)
      fact(:) = [ &
      5.126804285170776e+00, 2.269255340083806e+00, 5.042093004533025e-01, 4.569239924162220e-02, 1.886750504938862e-04,   &
      1.315815010261290e+01, 1.181295082433371e+00, 2.771070382190629e-01, 2.979576045495446e-02, 6.940110991990983e-04]
   case(63)
      fact(:) = [ &
      4.979623597498092e+00, 2.241830874556695e+00, 5.129339614028937e-01, 4.298018484983344e-02, 1.813816924857876e-04,   &
      1.283926630780338e+01, 1.147054464204860e+00, 2.703871609352437e-01, 2.824187149526020e-02, 6.714866031739924e-04]
   case(64)
      fact(:) = [ &
      5.078358300456114e+00, 1.957440271816590e+00, 5.928259832213751e-01, 4.195020341439256e-02, 1.752410915283168e-04,   &
      1.051327254690473e+01, 1.117649412815246e+00, 2.843867418336752e-01, 2.726633276413471e-02, 6.503108607073037e-04]
   case(65)
      fact(:) = [ &
      4.711616366573001e+00, 2.172619507865789e+00, 5.397176013428657e-01, 3.797960045478111e-02, 1.669237635648417e-04,   &
      1.231094159485391e+01, 1.087437967674925e+00, 2.598049655092755e-01, 2.532899238951897e-02, 6.292785669619112e-04]
   case(66)
      fact(:) = [ &
      4.590755044851466e+00, 2.135723730365674e+00, 5.513555600941525e-01, 3.560584067185138e-02, 1.598240168567898e-04,   &
      1.206567406233699e+01, 1.058117371158644e+00, 2.539944389186618e-01, 2.395087396496492e-02, 6.094827484332881e-04]
   case(67)
      fact(:) = [ &
      4.484001106267108e+00, 2.089043470785398e+00, 5.659326183161046e-01, 3.327035902110630e-02, 1.524456102828921e-04,   &
      1.187492506915706e+01, 1.028284937087896e+00, 2.489078079080760e-01, 2.258440655321478e-02, 5.903744451594245e-04]
   case(68)
      fact(:) = [ &
      4.376651407869632e+00, 2.046451503836341e+00, 5.806628605915877e-01, 3.123885283277667e-02, 1.453748696625357e-04,   &
      1.166530807171395e+01, 1.003147696754953e+00, 2.441532926863124e-01, 2.136185800007310e-02, 5.721103384547666e-04]
   case(69)
      fact(:) = [ &
      4.283083182474195e+00, 1.995380014537307e+00, 5.970535713325043e-01, 2.909516688695599e-02, 1.380647690375216e-04,   &
      1.149619059566834e+01, 9.770391020188206e-01, 2.394291329758291e-01, 2.009122333272558e-02, 5.543771384922663e-04]
   case(70)
      fact(:) = [ &
      4.195638407371233e+00, 1.943332859786456e+00, 6.124466172854658e-01, 2.715152076949324e-02, 1.305947873519297e-04,   &
      1.141077504566523e+01, 9.490119244548774e-01, 2.349503265353047e-01, 1.890515762809755e-02, 5.372336492097111e-04]
   case(71)
      fact(:) = [ &
      4.356925932963843e+00, 1.695892047773159e+00, 6.639045200684705e-01, 2.630200187602814e-02, 1.254973184998238e-04,   &
      9.294345147185689e+00, 9.105000459574200e-01, 2.387465959113764e-01, 1.820985425464623e-02, 5.215593719635622e-04]
   case(72)
      fact(:) = [ &
      4.331384056649235e+00, 1.527648647286807e+00, 7.357959228912380e-01, 2.495263232014026e-02, 1.187408525810578e-04,   &
      7.876844338102884e+00, 9.425156426277488e-01, 2.416994800398060e-01, 1.728998944485090e-02, 5.058346313135256e-04]
   case(73)
      fact(:) = [ &
      4.197260013196420e+00, 1.468548067747088e+00, 7.839312482110882e-01, 2.324940948643951e-02, 1.112613586072860e-04,   &
      6.936740248944572e+00, 1.017252766183483e+00, 2.389489397150706e-01, 1.623324330753810e-02, 4.902390040211490e-04]
   case(74)
      fact(:) = [ &
      3.976297158190774e+00, 1.522926842573418e+00, 7.977062463905613e-01, 2.136667415586852e-02, 1.030786893857173e-04,   &
      6.296857792287749e+00, 1.112899511576911e+00, 2.310570406784734e-01, 1.510135455672045e-02, 4.746824771050258e-04]
   case(75)
      fact(:) = [ &
      3.751443814398119e+00, 1.627688029776327e+00, 7.817567554542718e-01, 1.951708039122916e-02, 9.431998042934275e-05,   &
      5.797546360829538e+00, 1.182236310777107e+00, 2.199135860247702e-01, 1.398104554814566e-02, 4.591271258298386e-04]
   case(76)
      fact(:) = [ &
      3.484015173387116e+00, 1.793779204169655e+00, 7.448783566919203e-01, 1.754277851624345e-02, 8.448723515688600e-05,   &
      5.439988460809537e+00, 1.227921341401281e+00, 2.062088569929092e-01, 1.278994017211460e-02, 4.430322267874786e-04]
   case(77)
      fact(:) = [ &
      1.599565781988442e+00, 2.975344521941205e+00, 6.950926783668822e-01, 1.487796274586880e-02, 6.905495791015833e-05,   &
      5.792444473855675e+00, 1.553009829732260e+00, 1.886263359366482e-01, 1.117634706624533e-02, 4.227723616555514e-04]
   case(78)
      fact(:) = [ &
      2.040215639757283e+00, 2.899226346248255e+00, 6.363440838157977e-01, 1.320719195954083e-02, 5.673821925001312e-05,   &
      6.658194296096275e+00, 1.413379237789324e+00, 1.740001045021790e-01, 1.006878045302105e-02, 4.030771056922231e-04]
   case(79)
      fact(:) = [ &
      1.675934670648708e+00, 3.004866029697293e+00, 5.953400131616355e-01, 1.171631866230948e-02, 4.296782976398171e-05,   &
      5.522310932114025e+00, 1.380072230071963e+00, 1.622292376559454e-01, 9.018148904165756e-03, 3.792776674776671e-04]
   case(80)
      fact(:) = [ &
      2.235228504431053e+00, 2.682766386519949e+00, 5.551949262124333e-01, 1.072733543662587e-02, 3.284739920462629e-05,   &
      5.020309889602403e+00, 1.230775905837777e+00, 1.522481229928636e-01, 8.283991169062104e-03, 3.562419389317589e-04]
   case(81)
      fact(:) = [ &
      2.803427374125100e+00, 2.718827879660200e+00, 5.224759153919996e-01, 9.845378369710420e-03, 2.345245265083162e-05,   &
      6.558768728045342e+00, 1.169724225166679e+00, 1.435566918001781e-01, 7.619765262300398e-03, 3.296276739029854e-04]
   case(82)
      fact(:) = [ &
      3.608610209977714e+00, 2.450567747371983e+00, 4.786395001072570e-01, 8.872142351692152e-03, 1.040019329094673e-05,   &
      6.581625946219628e+00, 1.027728526605885e+00, 1.335336806347209e-01, 6.848612389484298e-03, 2.763888754566660e-04]
   case(83)
      fact(:) = [ &
      4.242099010573159e+00, 2.099943420558279e+00, 4.328366313799229e-01, 8.020215703810592e-03, 7.217850308513403e-07,   &
      5.752801155360800e+00, 8.739014891954615e-01, 1.235999561805247e-01, 6.176003330585702e-03, 1.414951776172309e-04]
   case(84)
      fact(:) = [ &
      4.636200956689623e+00, 1.780633114269871e+00, 4.037304439527385e-01, 7.685395546686740e-03, 8.954159028076436e-08,   &
      4.888252997809829e+00, 7.563105268800747e-01, 1.173023644522492e-01, 5.930343942428942e-03, 7.666686638730287e-05]
   case(85)
      fact(:) = [ &
      4.965922505950705e+00, 1.438155615070337e+00, 3.712992005792981e-01, 7.472564284502011e-03, 1.141152841525851e-07,   &
      4.091293787874963e+00, 6.292289966025513e-01, 1.110966786955352e-01, 5.772350103477618e-03, 8.085118938793825e-05]
   case(86)
      fact(:) = [ &
      5.306156144750332e+00, 1.117331602360721e+00, 3.158587231108543e-01, 7.203422423102100e-03, 1.073553305458635e-07,   &
      3.488007354816557e+00, 4.811907411497712e-01, 1.018744679457531e-01, 5.592453744170787e-03, 7.795066735407367e-05]
   case(87)
      fact(:) = [ &
      4.520533990423360e+00, 4.106953979091246e+00, 7.139468785037285e-01, 1.692940276874539e-02, 8.574921291917244e-05,   &
      1.944822342329488e+01, 1.898246731559969e+00, 1.695535635953418e-01, 1.148195675489342e-02, 3.461220382579827e-04]
   case(88)
      fact(:) = [ &
      6.524010720018733e+00, 3.207870807456613e+00, 5.404787743496377e-01, 8.782788068988532e-03, 6.910106029491315e-06,   &
      1.400925542989749e+01, 1.326350359616653e+00, 1.314085683860361e-01, 6.286474345204096e-03, 2.278399814589510e-04]
   case(89)
      fact(:) = [ &
      6.896028535596191e+00, 2.835141545365232e+00, 5.035068818888151e-01, 8.022945109667069e-03, 9.204009547587397e-08,   &
      1.107638256703987e+01, 1.171326162905030e+00, 1.234946513917999e-01, 5.735155957904971e-03, 7.197075418025232e-05]
   case(90)
      fact(:) = [ &
      7.093749001626302e+00, 2.529123739031940e+00, 4.821078198888890e-01, 7.719366741451110e-03, 7.271141994250414e-08,   &
      9.094737951659447e+00, 1.063916670331612e+00, 1.186194466218779e-01, 5.539457088950347e-03, 6.584947305284627e-05]
   case(91)
      fact(:) = [ &
      6.434013247972842e+00, 2.970999705357888e+00, 4.607966517878480e-01, 7.240331257754553e-03, 6.362366643091279e-08,   &
      1.025968513072979e+01, 1.131774523061633e+00, 1.127759353623823e-01, 5.274595833531321e-03, 6.192638240887597e-05]
   case(92)
      fact(:) = [ &
      6.210708267840199e+00, 3.039344538256235e+00, 4.373399844391628e-01, 6.807147641473166e-03, 6.182293277428513e-08,   &
      1.002141058591628e+01, 1.103998801558834e+00, 1.072189731624539e-01, 5.024321308506824e-03, 6.005069941971958e-05]
   case(93)
      fact(:) = [ &
      6.004315983089865e+00, 3.094316545314180e+00, 4.128084877164170e-01, 6.408916578800669e-03, 6.730073762160754e-08,   &
      9.817930467191061e+00, 1.069787050680294e+00, 1.016352914486458e-01, 4.795248468887250e-03, 6.028065987264629e-05]
   case(94)
      fact(:) = [ &
      5.200617168103042e+00, 3.498494403671440e+00, 4.054149311313203e-01, 6.223437008265299e-03, 6.008593295434409e-08,   &
      1.120383101944067e+01, 1.128463695469282e+00, 9.893334726861316e-02, 4.659174319779507e-03, 5.725906389623341e-05]
   case(95)
      fact(:) = [ &
      5.025338600360291e+00, 3.518439882851541e+00, 3.819503494462722e-01, 5.821102080962110e-03, 6.526093388297119e-08,   &
      1.097721906976287e+01, 1.084772148326308e+00, 9.367468748538359e-02, 4.426823468734856e-03, 5.738568370505115e-05]
   case(96)
      fact(:) = [ &
      5.346561002606197e+00, 3.224684665609108e+00, 3.494617526254452e-01, 5.292517567488151e-03, 6.159176302362374e-08,   &
      9.231183797524494e+00, 9.728362291938352e-01, 8.705962209665623e-02, 4.130119644650323e-03, 5.494638944496757e-05]
   case(97)
      fact(:) = [ &
      5.225823503340047e+00, 3.228188739952953e+00, 3.270988788238514e-01, 4.888825755922390e-03, 5.212722694193633e-08,   &
      9.071357066187183e+00, 9.311242774566348e-01, 8.207206020598455e-02, 3.890296559934166e-03, 5.103679766366416e-05]
   case(98)
      fact(:) = [ &
      4.586412477535479e+00, 3.518695956651012e+00, 3.192617142012054e-01, 4.729796479856258e-03, 5.513244808062041e-08,   &
      1.031861020923352e+01, 9.572788378292328e-01, 7.968074314438807e-02, 3.771931999968489e-03, 5.097508603760111e-05]
   case(99)
      fact(:) = [ &
      4.457994806754126e+00, 3.508672126376623e+00, 3.013753805283154e-01, 4.407637462144819e-03, 4.887879032285916e-08,   &
      1.008906933834013e+01, 9.194464473616226e-01, 7.564191777863293e-02, 3.570264651248093e-03, 4.804951743958113e-05]
   case(100)
      fact(:) = [ &
      4.338975764011709e+00, 3.491850988964034e+00, 2.814710990162868e-01, 4.002092714990461e-03, 5.529298081955140e-08,   &
      9.963309372238362e+00, 8.780839569829744e-01, 7.116371157687776e-02, 3.321524042150777e-03, 4.851031794603728e-05]
   case(101)
      fact(:) = [ &
      4.227294704031012e+00, 3.472492275106616e+00, 2.648222294968986e-01, 3.690728778331929e-03, 6.258714262002473e-08,   &
      9.724006020400314e+00, 8.428737759602104e-01, 6.735347439748511e-02, 3.123646062633208e-03, 4.912170970176192e-05]
   case(102)
      fact(:) = [ &
      4.109517024430204e+00, 3.457991325227507e+00, 2.470873512223867e-01, 3.304239209409952e-03, 5.991049262319316e-08,   &
      9.677359945101202e+00, 8.069400424708172e-01, 6.328154369541671e-02, 2.875446396412092e-03, 4.706791536975981e-05]
   case(103)
      fact(:) = [ &
      4.521474211983788e+00, 3.202129855878044e+00, 2.230287269564517e-01, 2.817164538920297e-03, 4.064279540386205e-08,   &
      8.283099068611421e+00, 7.319189581253939e-01, 5.809427730186550e-02, 2.561680160474449e-03, 4.038165155290065e-05]
   case default
      ier = 1
      fact(:) = 0
   end select

   end subroutine get_lobato_params

!----------------------------------------------------------------------------------------------------------------

   subroutine get_doyle_turner_params(zval,charge,fact,ier)
!
!  P. A. Doyle and P. S. Turner, Acta Cryst. (1968). A24, 390
!  Relativistic Hartree-Foek X-ray and Electron Scattering Factors
!
   use type_constants, only: DP
   integer, intent(in)                        :: zval
   integer, intent(in)                        :: charge
   real(DP), dimension(NPARDT), intent(out)   :: fact
   integer, intent(out)                       :: ier
!
   ier = 0
!
!  Order of parameters:
!                        a1       b1       a2       b2       a3       b3       a4       b4
   select case(zval)
   case(1)   !H      
      fact(:) =     [  0.3754, 15.4946,  0.1408,  4.1261,  0.0216,  0.0246, -0.1012, 46.8840]
   case(2)   !He     
      fact(:) =     [  0.0906, 18.1834,  0.1814,  6.2109,  0.1095,  1.8026,  0.0362,  0.2844]
   case(3)   !Li      
      if (charge == 0) then
          fact(:) = [  1.6108,107.6384,  1.2460, 30.4795,  0.3257,  4.5331,  0.0986,  0.4951]
      elseif (charge == 1) then
          fact(:) = [  0.0538,  1.0860,  0.0208,  7.6853,  0.0587,  3.1378,  0.0237,  0.1923]
      else
          ier = 2
      endif
   case(4)   !Be     
      fact(:) =     [  1.2498, 60.8042,  1.3335, 18.5914,  0.3603,  3.6534,  0.1055,  0.4157]
   case(5)   !B      
      fact(:) =     [  0.9446, 46.4438,  1.3120, 14.1778,  0.4188,  3.2228,  0.1159,  0.3767]
   case(6)   !C      
      fact(:) =     [  0.7307, 36.9951,  1.1951, 11.2966,  0.4563,  2.8139,  0.1247,  0.3456]
   case(7)   !N      
      fact(:) =     [  0.5717, 28.8465,  1.0425,  9.0542,  0.4647,  2.4213,  0.1311,  0.3167]
   case(8)   !O      
      fact(:) =     [  0.4548, 23.7803,  0.9173,  7.6220,  0.4719,  2.1440,  0.1384,  0.2959]
   case(9)   !F      
      fact(:) =     [  0.3686, 20.2390,  0.8109,  6.6093,  0.4751,  1.9310,  0.1459,  0.2793]
   case(10)   !Ne     
      fact(:) =     [  0.3025, 17.6396,  0.7202,  5.8604,  0.4751,  1.7623,  0.1534,  0.2656]
   case(11)   !Na      
      if (charge == 0) then
          fact(:) = [  2.2406,108.0039,  1.3326, 24.5047,  0.9070,  3.3914,  0.2863,  0.4346]
      elseif (charge == 1) then
          fact(:) = [  0.1677, 11.2946,  0.4740,  4.2735,  0.3590,  1.4135,  0.1289,  0.2262]
      else
          ier = 2
      endif
   case(12)   !Mg     
      fact(:) =     [  2.2682, 73.6704,  1.8025, 20.1749,  0.8394,  3.0181,  0.2892,  0.4046]
   case(13)   !Al     
      fact(:) =     [  2.2756, 72.3220,  2.4280, 19.7729,  0.8578,  3.0799,  0.3166,  0.4076]
   case(14)   !Si     
      fact(:) =     [  2.1293, 57.7748,  2.5333, 16.4756,  0.8349,  2.8796,  0.3216,  0.3860]
   case(15)   !P      
      fact(:) =     [  1.8882, 44.8756,  2.4685, 13.5383,  0.8046,  2.6424,  0.3204,  0.3608]
   case(16)   !S      
      fact(:) =     [  1.6591, 36.6500,  2.3863, 11.4881,  0.7899,  2.4686,  0.3208,  0.3403]
   case(17)   !Cl     
      if (charge == 0) then
          fact(:) = [  1.4524, 30.9352,  2.2926,  9.9798,  0.7874,  2.3336,  0.3217,  0.3228]
      elseif (charge == -1) then
          fact(:) = [  1.9290, 50.1174,  3.2667, 13.4759,  1.1666,  3.0567,  0.3972,  0.3713]
      else
          ier = 2
      endif
   case(18)   !Ar     
      fact(:) =     [  1.2736, 26.6823,  2.1894,  8.8130,  0.7927,  2.2186,  0.3225,  0.3071]
   case(19)   !K       
      if (charge == 0) then
          fact(:) = [  3.9507,137.0748,  2.5452, 22.4017,  1.9795,  4.5319,  0.4817,  0.4340]
      elseif (charge == 1) then
          fact(:) = [  0.8880, 18.3273,  1.6382,  6.7637,  0.6271,  1.8497,  0.2816,  0.2713]
      else
          ier = 2
      endif
   case(20)   !Ca     
      fact(:) =     [  4.4696, 99.5228,  2.9708, 22.6958,  1.9696,  4.1954,  0.4818,  0.4165]
   case(21)   !Sc     
      fact(:) =     [  3.9659, 88.9597,  2.9169, 20.6061,  1.9254,  3.8557,  0.4802,  0.3988]
   case(22)   !Ti     
      fact(:) =     [  3.5653, 81.9821,  2.8181, 19.0486,  1.8930,  3.5904,  0.4825,  0.3855]
   case(23)   !V      
      if (charge == 0) then
          fact(:) = [  3.2449, 76.3789,  2.6978, 17.7262,  1.8597,  3.3632,  0.4864,  0.3743]
      elseif (charge == 2) then
          fact(:) = [  0.4754, 19.2254,  1.3279,  5.9361,  0.7998,  1.8605,  0.2996,  0.2538]
      else
          ier = 2
      endif
   case(24)   !Cr     
      fact(:) =     [  2.3066, 78.4051,  2.3339, 15.7851,  1.8226,  3.1566,  0.4901,  0.3636]
   case(25)   !Mn     
      if (charge == 0) then
          fact(:) = [  2.7467, 67.7862,  2.4556, 15.6743,  1.7923,  2.9998,  0.4984,  0.3569]
      elseif (charge == 2) then
          fact(:) = [  0.4523, 17.3232,  1.2228,  5.4583,  0.8599,  1.7444,  0.3102,  0.2422]
      else
          ier = 2
      endif
   case(26)   !Fe     
      fact(:) =     [  2.5440, 64.4244,  2.3434, 14.8806,  1.7588,  2.8539,  0.5062,  0.3502]
   case(27)   !Co     
      if (charge == 0) then
          fact(:) = [  2.3668, 61.4306,  2.2361, 14.1798,  1.7243,  2.7247,  0.5148,  0.3442]
      elseif (charge == 2) then
          fact(:) = [  0.4149, 15.6581,  1.1218,  5.0435,  0.8959,  1.6265,  0.3208,  0.2323]
      else
          ier = 2
      endif
   case(28)   !Ni     
      fact(:) =     [  2.2104, 58.7267,  2.1342, 13.5530,  1.6891,  2.6094,  0.5238,  0.3388]
   case(29)   !Cu     
      if (charge == 0) then
          fact(:) = [  1.5792, 62.9403,  1.8197, 12.4527,  1.6576,  2.5042,  0.5323,  0.3331]
      elseif (charge == 1) then
          fact(:) = [  0.5401, 20.5410,  1.2368,  5.9307,  1.1176,  1.7355,  0.3843,  0.2501]
      else
          ier = 2
      endif
   case(30)   !Zn     
      fact(:) =     [  1.9418, 54.1621,  1.9501, 12.5177,  1.6192,  2.4164,  0.5434,  0.3295]
   case(31)   !Ga     
      fact(:) =     [  2.3205, 65.6019,  2.4855, 15.4577,  1.6879,  2.5806,  0.5992,  0.3510]
   case(32)   !Ge     
      fact(:) =     [  2.4467, 55.8930,  2.7015, 14.3930,  1.6157,  2.4461,  0.6009,  0.3415]
   case(33)   !As     
      fact(:) =     [  2.3989, 45.7179,  2.7898, 12.8166,  1.5288,  2.2799,  0.5936,  0.3277]
   case(34)   !Se     
      fact(:) =     [  1.0204,  0.8567,  1.5699,  4.2086,  3.0250, 16.8080,  1.4417, 48.6300]
   case(35)   !Br     
      if (charge == 0) then
          fact(:) = [  2.1659, 33.8987,  2.9037, 10.4996,  1.3951,  2.0413,  0.5886,  0.3070]
      elseif (charge == -1) then
          fact(:) = [  2.8044, 52.9153,  4.1294, 13.8053,  1.7053,  2.5144,  0.7016,  0.3511]
      else
          ier = 2
      endif
   case(36)   !Kr     
      fact(:) =     [  2.0338, 29.9992,  2.9271,  9.5977,  1.3425,  1.9520,  0.5888,  0.2986]
   case(37)   !Rb      
      if (charge == 0) then
          fact(:) = [  4.7760,140.7821,  3.8588, 18.9910,  2.2339,  3.7010,  0.8683,  0.4194]
      elseif (charge == 1) then
          fact(:) = [  1.5652, 21.4061,  2.3080,  7.6576,  1.1424,  1.6782,  0.5270,  0.2693]
      else
          ier = 2
      endif
   case(38)   !Sr      
      fact(:) =     [  5.8478,104.9721,  4.0026, 19.3666,  2.3420,  3.7368,  0.8795,  0.4142]
   case(40)   !Zr      
      fact(:) =     [  1.3014,  0.9856,  3.2282,  6.6214,  4.0636, 30.6316,  3.3620,101.1290]
   case(42)   !Mo     
      fact(:) =     [  3.1199, 72.4642,  3.9061, 14.6424,  2.3615,  3.2370,  0.8504,  0.3662]
   case(47)   !Ag     
      fact(:) =     [  2.0355, 61.4970,  3.2716, 11.8237,  2.5105,  2.8456,  0.8372,  0.3271]
   case(48)   !Cd     
      fact(:) =     [  2.5737, 55.6752,  3.2586, 11.8376,  2.5468,  2.7842,  0.8379,  0.3217]
   case(49)   !In     
      fact(:) =     [  3.1528, 66.6492,  3.5565, 14.4494,  2.8180,  2.9758,  0.8842,  0.3345]
   case(50)   !Sn     
      if (charge == 0) then
          fact(:) = [  3.4495, 59.1042,  3.7349, 14.1787,  2.7779,  2.8548,  0.8786,  0.3270]
      elseif (charge == 2) then
          fact(:) = [  1.4363, 25.5762,  2.2527,  6.9676,  1.7685,  2.0370,  0.6836,  0.2641]
      elseif (charge == 4) then
          fact(:) = [  0.6327, 10.3654,  1.9013,  4.0965,  1.0063,  1.3935,  0.5307,  0.2129]
      else
          ier = 2
      endif
   case(51)   !Sb     
      fact(:) =     [  3.5644, 50.4869,  3.8437, 13.3156,  2.6866,  2.6909,  0.8638,  0.3161]
   case(53)   !I      
      if (charge == 0) then
          fact(:) = [  3.4728, 39.4411,  4.0602, 11.8161,  2.5215,  2.4148,  0.8398,  0.2976]
      elseif (charge == -1) then
          fact(:) = [  4.2495, 59.6448,  5.6512, 15.5905,  2.9603,  2.7837,  0.9489,  0.3300]
      else
          ier = 2
      endif
   case(54)   !Xe     
      fact(:) =     [  3.3656, 35.5094,  4.1468, 11.1170,  2.4430,  2.2940,  0.8293,  0.2892]
   case(55)   !Cs      
      if (charge == 0) then
          fact(:) = [  6.0620,155.8336,  5.9861, 19.6951,  3.3033,  3.3354,  1.0958,  0.3793]
      elseif (charge == 1) then
          fact(:) = [  2.7761, 26.0552,  3.3558,  9.0139,  2.1398,  2.0273,  0.7588,  0.2654]
      else
          ier = 2
      endif
   case(56)   !Ba      
      fact(:) =     [  7.8212,117.6575,  6.0040, 18.7782,  3.2803,  3.2634,  1.1030,  0.3760]
   case(63)   !Eu      
      fact(:) =     [  6.2667,100.2983,  4.8440, 16.0662,  3.2023,  2.9803,  1.2009,  0.3674]
   case(73)   !Ta     
      fact(:) =     [  1.9885,  0.8784,  3.4824,  5.1259,  4.0794, 23.4938,  2.9841, 80.1163]
   case(79)   !Au     
      fact(:) =     [  2.3880, 42.8656,  4.2259,  9.7430,  2.6886,  2.2641,  1.2551,  0.3067]
   case(80)   !Hg     
      fact(:) =     [  2.6817, 42.8217,  4.2414,  9.8557,  2.7549,  2.2951,  1.2708,  0.3067]
   case(82)   !Pb     
      fact(:) =     [  3.5099, 52.9141,  4.5523, 11.8840,  3.1539,  2.5713,  1.3591,  0.3205]
   case(83)   !Bi     
      fact(:) =     [  3.8412, 50.2608,  4.6784, 11.9988,  3.1924,  2.5598,  1.3625,  0.3177]
   case(86)   !Rn     
      fact(:) =     [  4.0779, 28.4058,  4.9778, 11.0204,  3.0955,  2.3549,  1.3259,  0.2991]
   case(92)   !U      
      fact(:) =     [  6.7668, 85.9510,  6.7287, 15.6415,  4.0135,  2.9364,  1.5607,  0.3348]
   case default
      ier = 1
   end select

   end subroutine get_doyle_turner_params

#if 0
!----------------------------------------------------------------------------------------------------------------

   subroutine make_kirkland_params(filename,err)
   use fileutil
   use errormod
   use strutil
   use iso_fortran_env, only: stdout => output_unit
   character(len=*), intent(in)                :: filename
   type(error_type), intent(out)               :: err
   type(file_handle)                           :: filep
   integer                                     :: ier, nline, nword
   character(len=:), allocatable               :: line
   character(len=:), allocatable, dimension(:) :: wordv
   integer                                     :: nelem
   integer, parameter                          :: NMAX_ELEM = 103
!
   call filep%fopen(filename)
   if (filep%fail()) then
       call err%set(filep%err_msg())
       return
   endif
!
!  Find first occurance of 'Z='
   nline = 0
   call find_key_file_a(filep%handle(),'Z=',nline,line,ier)
   call set_record_pos(filep%handle(),nline-1)
!
   write(stdout,'(3x,a)')'subroutine get_kirkland_params(zval,fact,ier)'
   write(stdout,'(a)')'!'
   write(stdout,'(a)')'!  Kirkland, E. J. (1998). Advanced Computing in Electron Microscopy.'
   write(stdout,'(a)')'!  New York: Springer. Appendix D and Chapter 5.'
   write(stdout,'(a)')'!'
   write(stdout,'(3x,a)')'use type_constants, only: DP'
   write(stdout,'(3x,a)')'integer, intent(in)                        :: zval'
   write(stdout,'(3x,a)')'real(DP), dimension(NPARKIRK), intent(out) :: fact'
   write(stdout,'(3x,a)')'integer, intent(out)                       :: ier'
   write(stdout,'(a/3x,a/a)')   '!','ier = 0','!'
   write(stdout,'(a)')   '!  Order of parameters:'
   write(stdout,'(a)')   '!           a1              b1               a2               b2'
   write(stdout,'(a)')   '!           a3              b3               c1               d1'
   write(stdout,'(a)')   '!           c2              d2               c3               d3'
   write(stdout,'(3x,a)')'select case(zval)'
   nelem = 0
   nline = 0
   do while(get_line(filep%handle(),line,trimmed=.true.,filter=.true.))
      if (len_trim(line) == 0) cycle
      if (index(line,'Z=') > 0) then
          nelem = nelem + 1
          nline = 0
          write(stdout,'(3x,a,i0,a)')'case(',nelem,')'
          write(stdout,'(6x,a)')'fact(:) = [ &'
      else
          nline = nline + 1
          call get_words1(line,wordv,nword)
          write(stdout,'(6x,a,a,3(1x,a,a))',advance='no')wordv(1),',',wordv(2),',',wordv(3),',',wordv(4)
          if (nline == 3) then
              write(stdout,'(a)',advance='yes')']'
          else
              write(stdout,'(a)',advance='yes')',   &'
          endif
      endif
   enddo
   write(stdout,'(3x,a)')'case default'
   write(stdout,'(3x,a)')'   ier = 1'
   write(stdout,'(3x,a)')'   fact(:) = 0'
   write(stdout,'(3x,a/)')'end select'
   write(stdout,'(3x,a)') 'end subroutine get_kirkland_params'
!
   end subroutine make_kirkland_params

!----------------------------------------------------------------------------------------------------------------

   subroutine make_lobato_params(filename,err)
   use fileutil
   use errormod
   use strutil
   use iso_fortran_env, only: stdout => output_unit
   character(len=*), intent(in)                :: filename
   type(error_type), intent(out)               :: err
   type(file_handle)                           :: filep
   integer                                     :: ier, nline, nword
   character(len=:), allocatable               :: line
   character(len=:), allocatable, dimension(:) :: wordv
   integer                                     :: nelem
   integer, parameter                          :: NMAX_ELEM = 103
!
   call filep%fopen(filename)
   if (filep%fail()) then
       call err%set(filep%err_msg())
       return
   endif
!
!  Find first occurance of 'Z='
   nline = 0
   call find_key_file_a(filep%handle(),'Z =',nline,line,ier)
   call set_record_pos(filep%handle(),nline-1)
!
   write(stdout,'(3x,a)')'subroutine get_lobato_params(zval,fact,ier)'
   write(stdout,'(a)')'!'
   write(stdout,'(a)')'!  I. Lobato and D. Van Dyck, Acta Cryst. (2014). A70, 636-649'
   write(stdout,'(a)')'!  An accurate parameterization for scattering factors,'
   write(stdout,'(a)')'!  electron densities and electrostatic potentials for'
   write(stdout,'(a)')'!  neutral atoms that obey all physical constraints'
   write(stdout,'(a)')'!'
   write(stdout,'(3x,a)')'use type_constants, only: DP'
   write(stdout,'(3x,a)')'integer, intent(in)                        :: zval'
   write(stdout,'(3x,a)')'real(DP), dimension(NPARLOBA), intent(out) :: fact'
   write(stdout,'(3x,a)')'integer, intent(out)                       :: ier'
   write(stdout,'(a/3x,a/a)')   '!','ier = 0','!'
   write(stdout,'(a)')   '!  Order of parameters:'
   write(stdout,'(a)')   '!           a1    a2    a3    a4    a5'
   write(stdout,'(a)')   '!           b1    b2    b3    b4    b5'
   write(stdout,'(3x,a)')'select case(zval)'
   nelem = 0
   nline = 0
   do while(get_line(filep%handle(),line,trimmed=.true.,filter=.true.))
      if (len_trim(line) == 0) cycle
      if (index(line,'Z =') > 0) then
          nelem = nelem + 1
          nline = 0
          write(stdout,'(3x,a,i0,a)')'case(',nelem,')'
          write(stdout,'(6x,a)')'fact(:) = [ &'
      else
          nline = nline + 1
          call get_words1(line,wordv,nword)
          write(stdout,'(6x,a,a,4(1x,a,a))',advance='no')trim(wordv(1)),',',trim(wordv(2)),',',   &
                                                         trim(wordv(3)),',',trim(wordv(4)),',',trim(wordv(5))
          if (nline == 2) then
              write(stdout,'(a)',advance='yes')']'
          else
              write(stdout,'(a)',advance='yes')',   &'
          endif
      endif
   enddo
   write(stdout,'(3x,a)')'case default'
   write(stdout,'(3x,a)')'   ier = 1'
   write(stdout,'(3x,a)')'   fact(:) = 0'
   write(stdout,'(3x,a/)')'end select'
   write(stdout,'(3x,a)') 'end subroutine get_lobato_params'

   end subroutine make_lobato_params  

!----------------------------------------------------------------------------------------------------------------

   subroutine make_doyle_turner_params(filename1,filename2,err)
   use fileutil
   use errormod
   use strutil
   use iso_fortran_env, only: stdout => output_unit
   character(len=*), intent(in)                :: filename1,filename2
   type(error_type), intent(out)               :: err
   type(file_handle)                           :: filep1,filep2
   integer                                     :: ier, nline, nword, nword2
   character(len=:), allocatable               :: line, line2
   character(len=:), allocatable, dimension(:) :: wordv1,wordv2
   integer                                     :: i
   logical :: is_ion
!
   call filep1%fopen(filename1)
   call filep2%fopen(filename2)
   if (filep1%fail() .or. filep2%fail()) then
       call err%set(filep1%err_msg())
       return
   endif

   write(stdout,'(3x,a)')'subroutine get_doyle_turner_params(zval,charge,fact,ier)'
   write(stdout,'(a)')'!'
   write(stdout,'(a)')'!  P. A. Doyle and P. S. Turner, Acta Cryst. (1968). A24, 390'
   write(stdout,'(a)')'!  Relativistic Hartree-Foek X-ray and Electron Scattering Factors'
   write(stdout,'(a)')'!'
   write(stdout,'(3x,a)')'use type_constants, only: DP'
   write(stdout,'(3x,a)')'integer, intent(in)                        :: zval'
   write(stdout,'(3x,a)')'integer, intent(in)                        :: charge'
   write(stdout,'(3x,a)')'real(DP), dimension(NPARDT), intent(out)   :: fact'
   write(stdout,'(3x,a)')'integer, intent(out)                       :: ier'
   write(stdout,'(a/3x,a/a)')   '!','ier = 0','!'
   write(stdout,'(a)')   '!  Order of parameters:'
   write(stdout,'(a)')   '!                        a1       b1       a2       b2       a3       b3       a4       b4'
   write(stdout,'(3x,a)')'select case(zval)'

   do while(get_line(filep1%handle(),line,trimmed=.true.,filter=.true.))
      if (len_trim(line) == 0) cycle
      if (is_comment_line(line)) cycle

      call get_words1(line,wordv1,nword)
      write(stdout,'(3x,a,a,a,a)')'case(',trim(adjustl(wordv1(1))),')   !',wordv1(2)

      is_ion = .false.
      do while(get_line(filep2%handle(),line2,trimmed=.true.,filter=.true.))
         call get_words1(line2,wordv2,nword2)
         if (wordv1(1) == wordv2(1)) then
             is_ion = .true.
         endif
      enddo
      call filep2%rew()

      if (is_ion) then
          write(stdout,'(6x,a)') 'if (charge == 0) then'
          write(stdout,'(10x,a,7(a8,","))',advance='no')    &
                       'fact(:) = [',[(adjustr(wordv1(i)),i=3,nword-1)]
          write(stdout,'(a8,"]")')adjustr(wordv1(nword))
          do while(get_line(filep2%handle(),line2,trimmed=.true.,filter=.true.))
             call get_words1(line2,wordv2,nword2)
             if (wordv1(1) == wordv2(1)) then
                 write(stdout,'(6x,a,a,a)') 'elseif (charge == ',trim(wordv2(3)),') then'
                 write(stdout,'(10x,a,7(a8,","))',advance='no')    &
                              'fact(:) = [',[(adjustr(wordv2(i)),i=4,nword2-1)]
                 write(stdout,'(a8,"]")')adjustr(wordv2(nword2))
             endif
          enddo
          write(stdout,'(6x,a)') 'else'
          write(stdout,'(10x,a)') 'ier = 2'
          write(stdout,'(6x,a)') 'endif'
      else
          write(stdout,'(6x,a,7(a8,","))',advance='no')    &
                       'fact(:) =     [',[(adjustr(wordv1(i)),i=3,nword-1)]
          write(stdout,'(a8,"]")')adjustr(wordv1(nword))
      endif
   enddo

   write(stdout,'(3x,a)')'case default'
   write(stdout,'(3x,a)')'   ier = 1'
   write(stdout,'(3x,a/)')'end select'
   write(stdout,'(3x,a)') 'end subroutine get_doyle_turner_params'

   end subroutine make_doyle_turner_params
#endif
END MODULE SCATT_PARAMS
